#include "Countdown.h"

#include <math.h>
#include <iostream>

GLuint Countdown::Digits[10];
bool Countdown::TexturesLoaded = false;


Countdown::Countdown()
{
	TimeOut = 0;
	StartTime = 0;
	if (!TexturesLoaded)
	{
		loadTextures();
		TexturesLoaded = true;
	}
}


void Countdown::loadTextures()
{
	Digits[0] = SOIL_load_OGL_texture("../images/zero.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	Digits[1] = SOIL_load_OGL_texture("../images/one.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	Digits[2] = SOIL_load_OGL_texture("../images/two.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	Digits[3] = SOIL_load_OGL_texture("../images/three.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	Digits[4] = SOIL_load_OGL_texture("../images/four.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	Digits[5] = SOIL_load_OGL_texture("../images/five.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	Digits[6] = SOIL_load_OGL_texture("../images/six.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	Digits[7] = SOIL_load_OGL_texture("../images/seven.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	Digits[8] = SOIL_load_OGL_texture("../images/eight.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	Digits[9] = SOIL_load_OGL_texture("../images/nine.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
}

Countdown::~Countdown(void)
{
}


void Countdown::start(unsigned int timeout)
{
	TimeOut = timeout;
	StartTime = TimeFunctions::getTime();
}

bool Countdown::hasElapsed()
{
	if ( StartTime == 0 )
	{
		return true;
	}
	else
	{
		return ( (TimeFunctions::getTime() - StartTime) > (long) TimeOut);
	}
}

void Countdown::draw(float window_aspect)
{
	long current_time = TimeFunctions::getTime();
	int time_diff = current_time - StartTime;
	long remaining_millis = StartTime + TimeOut - current_time;
	int remaining_time = (remaining_millis) / 1000;
	float scaling = (float) (time_diff % 1000); // we scale every digit from small to large every 1000 milliseconds

	glDisable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix(); //save projection transformations
	glLoadIdentity();
	glOrtho(-1000.0*window_aspect, 1000.0*window_aspect, -1000.0, 1000.0, -10.0, 10.0);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix(); //save modelview transformations
	glLoadIdentity();

	glScalef(scaling, scaling, scaling);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f - scaling/1000.f ); // digit opacity in respect to scaling 

	// we calculate the length of the number...
	int length = (int) (log10f((float) remaining_time)) +1;

	// ... and determine the particular digits
	int *digits_to_show = new int(length);
	for (int i = 0; i < length; i++)
	{
		digits_to_show[i] = remaining_time % 10;
		remaining_time /= 10;
	}
	
	// show the number 
	// TODO: kerning
	glTranslatef((float) -length-1, 0.0f, 0.0f);
	glEnable(GL_TEXTURE_2D);
	for (int i = length-1; i >= 0; i--)
	{
		glBindTexture(GL_TEXTURE_2D, Digits[digits_to_show[i]]);
		glTranslatef(2.0f, 0.0f, 0.0f);
		glBegin(GL_QUADS);
			glTexCoord2f(0.0, 0.0);		glVertex3f(-1.0, 1.0, 0.0);
			glTexCoord2f(0.0, 1.0);		glVertex3f(-1.0, -1.0, 0.0);
			glTexCoord2f(1.0, 1.0);		glVertex3f(1.0, -1.0, 0.0);
			glTexCoord2f(1.0, 0.0);		glVertex3f(1.0, 1.0, 0.0);
		glEnd();
	}
	
	glDisable(GL_TEXTURE_2D);

	// clean memory
	delete[] digits_to_show;

	glPopMatrix(); //restore modelview transformations
	glMatrixMode(GL_PROJECTION);
	glPopMatrix(); //restore projection transformations
	glMatrixMode(GL_MODELVIEW);
	glEnable(GL_DEPTH_TEST);
}