#include "GestureUtility.h"
#include "math\Math.h"
#include "math\svd.h"
#include <GL\freeglut.h>
#include <assert.h>

// internally used when generating poses from skeleton data
static NUI_SKELETON_DATA last_skeleton;

// vectors from spine position to torso points
static cgv::math::mat<float> torso_vectors = cgv::math::ones<float>(3,6);


//helper functions
void smoothTorsoPositions(float strength, NUI_SKELETON_DATA &skeleton)
{
	skeleton.SkeletonPositions[0].x = strength * last_skeleton.SkeletonPositions[0].x + (1.0f - strength) * skeleton.SkeletonPositions[0].x;
	skeleton.SkeletonPositions[0].y = strength * last_skeleton.SkeletonPositions[0].y + (1.0f - strength) * skeleton.SkeletonPositions[0].y;
	skeleton.SkeletonPositions[0].z = strength * last_skeleton.SkeletonPositions[0].z + (1.0f - strength) * skeleton.SkeletonPositions[0].z;

	/*skeleton.SkeletonPositions[1].x = strength * last_skeleton.SkeletonPositions[1].x + (1.0f - strength) * skeleton.SkeletonPositions[1].x;
	skeleton.SkeletonPositions[1].y = strength * last_skeleton.SkeletonPositions[1].y + (1.0f - strength) * skeleton.SkeletonPositions[1].y;
	skeleton.SkeletonPositions[1].z = strength * last_skeleton.SkeletonPositions[1].z + (1.0f - strength) * skeleton.SkeletonPositions[1].z;*/

	// heavily smooth out the centroid point (spine)
	skeleton.SkeletonPositions[1].x = 0.9f * last_skeleton.SkeletonPositions[1].x + 0.1f * skeleton.SkeletonPositions[1].x;
	skeleton.SkeletonPositions[1].y = 0.9f * last_skeleton.SkeletonPositions[1].y + 0.1f * skeleton.SkeletonPositions[1].y;
	skeleton.SkeletonPositions[1].z = 0.9f * last_skeleton.SkeletonPositions[1].z + 0.1f * skeleton.SkeletonPositions[1].z;


	skeleton.SkeletonPositions[2].x = strength * last_skeleton.SkeletonPositions[2].x + (1.0f - strength) * skeleton.SkeletonPositions[2].x;
	skeleton.SkeletonPositions[2].y = strength * last_skeleton.SkeletonPositions[2].y + (1.0f - strength) * skeleton.SkeletonPositions[2].y;
	skeleton.SkeletonPositions[2].z = strength * last_skeleton.SkeletonPositions[2].z + (1.0f - strength) * skeleton.SkeletonPositions[2].z;

	skeleton.SkeletonPositions[4].x = strength * last_skeleton.SkeletonPositions[4].x + (1.0f - strength) * skeleton.SkeletonPositions[4].x;
	skeleton.SkeletonPositions[4].y = strength * last_skeleton.SkeletonPositions[4].y + (1.0f - strength) * skeleton.SkeletonPositions[4].y;
	skeleton.SkeletonPositions[4].z = strength * last_skeleton.SkeletonPositions[4].z + (1.0f - strength) * skeleton.SkeletonPositions[4].z;

	skeleton.SkeletonPositions[8].x = strength * last_skeleton.SkeletonPositions[8].x + (1.0f - strength) * skeleton.SkeletonPositions[8].x;
	skeleton.SkeletonPositions[8].y = strength * last_skeleton.SkeletonPositions[8].y + (1.0f - strength) * skeleton.SkeletonPositions[8].y;
	skeleton.SkeletonPositions[8].z = strength * last_skeleton.SkeletonPositions[8].z + (1.0f - strength) * skeleton.SkeletonPositions[8].z;

	skeleton.SkeletonPositions[12].x = strength * last_skeleton.SkeletonPositions[12].x + (1.0f - strength) * skeleton.SkeletonPositions[12].x;
	skeleton.SkeletonPositions[12].y = strength * last_skeleton.SkeletonPositions[12].y + (1.0f - strength) * skeleton.SkeletonPositions[12].y;
	skeleton.SkeletonPositions[12].z = strength * last_skeleton.SkeletonPositions[12].z + (1.0f - strength) * skeleton.SkeletonPositions[12].z;

	skeleton.SkeletonPositions[16].x = strength * last_skeleton.SkeletonPositions[16].x + (1.0f - strength) * skeleton.SkeletonPositions[16].x;
	skeleton.SkeletonPositions[16].y = strength * last_skeleton.SkeletonPositions[16].y + (1.0f - strength) * skeleton.SkeletonPositions[16].y;
	skeleton.SkeletonPositions[16].z = strength * last_skeleton.SkeletonPositions[16].z + (1.0f - strength) * skeleton.SkeletonPositions[16].z;

	last_skeleton = skeleton;
}


void setSkeletonPosition(NUI_SKELETON_DATA &skeleton, unsigned int index, cgv::math::vec<float> new_position)
{
	skeleton.SkeletonPositions[index].x = new_position[0];
	skeleton.SkeletonPositions[index].y = new_position[1];
	skeleton.SkeletonPositions[index].z = new_position[2];
}


inline math::Vec3f Vector4ToVec3f(const Vector4 &v)
{
	return math::Vec3f(v.x,v.y,v.z);
}


inline void clamp(float &input)
{
	if (input > 1.0f) input = 1.0f;
	if (input < -1.0f) input = -1.0f;
}


namespace GestureUtility
{
	void setPoseFromSkeletonData(NUI_SKELETON_DATA &skeleton, Pose &pose)
	{
		// we smooth out the torso points in time since there is typically not much overall movement
		smoothTorsoPositions(0.4f, skeleton);

		// vector upper arm left
		math::Vec3f upper_arm_left = Vector4ToVec3f(skeleton.SkeletonPositions[5]) -  Vector4ToVec3f(skeleton.SkeletonPositions[4]);
		// vector forearm left
		math::Vec3f forearm_left = Vector4ToVec3f(skeleton.SkeletonPositions[6]) -  Vector4ToVec3f(skeleton.SkeletonPositions[5]);
		// vector upper arm right
		math::Vec3f upper_arm_right = Vector4ToVec3f(skeleton.SkeletonPositions[9]) -  Vector4ToVec3f(skeleton.SkeletonPositions[8]);
		// vector forearm right
		math::Vec3f forearm_right = Vector4ToVec3f(skeleton.SkeletonPositions[10]) -  Vector4ToVec3f(skeleton.SkeletonPositions[9]);
		//vector spine
		math::Vec3f spine = Vector4ToVec3f(skeleton.SkeletonPositions[1]) -  Vector4ToVec3f(skeleton.SkeletonPositions[2]);
		//vector shoulder left
		math::Vec3f shoulder_left = Vector4ToVec3f(skeleton.SkeletonPositions[4]) -  Vector4ToVec3f(skeleton.SkeletonPositions[2]);
		//vector shoulder right
		math::Vec3f shoulder_right = Vector4ToVec3f(skeleton.SkeletonPositions[8]) -  Vector4ToVec3f(skeleton.SkeletonPositions[2]);
		//vector left hip
		math::Vec3f hip_left = Vector4ToVec3f(skeleton.SkeletonPositions[12]) -  Vector4ToVec3f(skeleton.SkeletonPositions[0]);
		// vector from shoulder to shoulder
		math::Vec3f shoulder_left_to_shoulder_right = Vector4ToVec3f(skeleton.SkeletonPositions[8]) -  Vector4ToVec3f(skeleton.SkeletonPositions[4]);
		//vector from hip to hip
		math::Vec3f hip_left_to_hip_right = Vector4ToVec3f(skeleton.SkeletonPositions[16]) -  Vector4ToVec3f(skeleton.SkeletonPositions[12]);

		// let's calculate the normals of the body planes from the seven torso points via SVD
		math::Vec3f normal_sagittal_plane(1,0,0); 
		math::Vec3f normal_transversal_plane(0,1,0); 
		math::Vec3f normal_coronal_plane(0,0,1); 
		
		cgv::math::mat<float> torso_points(3,7);
		
		torso_points.set_col( 0, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[0]).v) ); 
		torso_points.set_col( 1, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[1]).v) ); 
		torso_points.set_col( 2, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[2]).v) ); 
		torso_points.set_col( 3, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[4]).v) ); 
		torso_points.set_col( 4, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[8]).v) ); 
		torso_points.set_col( 5, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[12]).v) ); 
		torso_points.set_col( 6, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[16]).v) ); 
		
		// smooth out the torso vectors massively (we treat the torso as rigid)
		cgv::math::mat<float> new_torso_vectors(3,6);
		new_torso_vectors.set_col( 0, torso_points.col(0) - torso_points.col(1) );
		new_torso_vectors.set_col( 1, torso_points.col(2) - torso_points.col(1) );
		new_torso_vectors.set_col( 2, torso_points.col(3) - torso_points.col(1) );
		new_torso_vectors.set_col( 3, torso_points.col(4) - torso_points.col(1) );
		new_torso_vectors.set_col( 4, torso_points.col(5) - torso_points.col(1) );
		new_torso_vectors.set_col( 5, torso_points.col(6) - torso_points.col(1) );

		torso_vectors = torso_vectors * 0.9f + new_torso_vectors * 0.1f;

		// recalculate the skeleton positions from torso vectors
		setSkeletonPosition(skeleton, 0, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[1]).v) + torso_vectors.col(0) );
		setSkeletonPosition(skeleton, 2, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[1]).v) + torso_vectors.col(1) );
		setSkeletonPosition(skeleton, 4, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[1]).v) + torso_vectors.col(2) );
		setSkeletonPosition(skeleton, 8, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[1]).v) + torso_vectors.col(3) );
		setSkeletonPosition(skeleton, 12, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[1]).v) + torso_vectors.col(4) );
		setSkeletonPosition(skeleton, 16, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[1]).v) + torso_vectors.col(5) );

		// reset the current torso_points
		torso_points.set_col( 0, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[0]).v) ); 
		torso_points.set_col( 2, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[2]).v) ); 
		torso_points.set_col( 3, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[4]).v) ); 
		torso_points.set_col( 4, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[8]).v) ); 
		torso_points.set_col( 5, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[12]).v) ); 
		torso_points.set_col( 6, cgv::math::vec<float>(3,Vector4ToVec3f(skeleton.SkeletonPositions[16]).v) );

		//  calculate the means over rows of data set matrix
		cgv::math::vec<float> means(3);
		cgv::math::vec<float> idVec = cgv::math::ones<float>(7,1).col(0); //help vector of size 7, all entries set to one
		for (unsigned i = 0; i < 3; i++)
		{
			means[i] = cgv::math::dot(torso_points.row(i),idVec)/7.0f;
		}

		// subtract row mean from every entry in corresponding row
		for (int i = 0; i < 3; i++)
		{
			for (int j = 0; j < 7; j++)
			{
				torso_points(i,j) = torso_points(i,j) - means[i];
			}
		}
		
		// matrix is centered, let's do the SVD 	
		// svd of matrix Y = 1/sqrt(nr of cols - 1) * Data^T = U*S*V^T
		// V then contains the principal components of covariance matrix Data*Data^T (as column vectors)		
		cgv::math::mat<float> data;
		//data = 1.0f/(sqrtf((float)torso_points.ncols() - 1.0f))*cgv::math::transpose(torso_points); 
		data = cgv::math::transpose(torso_points); 

		cgv::math::diag_mat<float> s;
		s.resize(3);
		cgv::math::mat<float> principal_components;
		principal_components.resize(3,3);
	
		if(cgv::math::svd(data, data, s, principal_components,true,20))
		{
			// 1st principal component is y-axis
			normal_transversal_plane.x = principal_components.col(0)[0];
			normal_transversal_plane.y = principal_components.col(0)[1];
			normal_transversal_plane.z = principal_components.col(0)[2];
			normal_transversal_plane.normalize();
			// we have to ensure the orientation is from bottom of body to top of body
			math::Vec3f bottom_top = Vector4ToVec3f(skeleton.SkeletonPositions[3]) -  Vector4ToVec3f(skeleton.SkeletonPositions[0]);
			bottom_top.normalize();
			if(math::dotProduct(normal_transversal_plane, bottom_top) < 0.0f)
				normal_transversal_plane *= -1;

			// 2nd principal component is x-axis
			normal_sagittal_plane.x = principal_components.col(1)[0];
			normal_sagittal_plane.y = principal_components.col(1)[1];
			normal_sagittal_plane.z = principal_components.col(1)[2];
			normal_sagittal_plane.normalize();
			// we have to ensure the orientation is from left side of body to right side of body 
			math::Vec3f left_right = hip_left_to_hip_right;
			left_right.normalize();
			if(math::dotProduct(normal_sagittal_plane, left_right) < 0.0f)
				normal_sagittal_plane *= -1;

			// 3rd principal component is cross product of x- and y-axis
			normal_coronal_plane = math::crossProduct(normal_transversal_plane, normal_sagittal_plane);

			 //show coordinate system
			//glPushMatrix();
			//glTranslatef(skeleton.SkeletonPositions[0].x, skeleton.SkeletonPositions[0].y, skeleton.SkeletonPositions[0].z);
			////glScalef(.3f,.3f,.3f);
			//glBegin(GL_LINES);
			//	glColor3f(1.0,0.0,0.0);
			//	glVertex3f(0.0,0.0,0.0);
			//	glVertex3fv(normal_sagittal_plane.v);
			//
			//	glColor3f(0.0,1.0,0.0);
			//	glVertex3f(0.0,0.0,0.0);
			//	glVertex3fv(normal_transversal_plane.v);
			//
			//	glColor3f(0.0,0.0,1.0);
			//	glVertex3f(0.0,0.0,0.0);
			//	glVertex3fv(normal_coronal_plane.v);
			//glEnd();

			//glPopMatrix();
		}

		//normal of sagittal plane of lower part of body					<- not used anymore
		//math::Vec3f normal_lower_saggital_plane = hip_left_to_hip_right;
		//normal_lower_saggital_plane.normalize();
		
		//normal of transverse plane of lower part of body
		//math::Vec3f normal_lower_transverse_plane = - spine;
		//normal_lower_transverse_plane.normalize();

		//normal of coronal plane of lower part of body
		//math::Vec3f normal_lower_coronal_plane = math::crossProduct( normal_lower_transverse_plane, normal_lower_saggital_plane );
		//normal_lower_coronal_plane.normalize();
	
		// right hand distance in sagittal plane 
		// 1. get vector from hip center to right wrist (we don't use shoulder center because it can be occluded)
		math::Vec3f right_hand_spine = Vector4ToVec3f(skeleton.SkeletonPositions[10]) - Vector4ToVec3f(skeleton.SkeletonPositions[1]);
		// 2. to get distance we project vector onto normal of saggital plane (we use lower saggital plane because hand can occlude shoulder)
		float sagittal_distance_right_hand = math::dotProduct(right_hand_spine, normal_sagittal_plane);

		// right hand distance in transverse plane
		float transverse_distance_right_hand = math::dotProduct(right_hand_spine, normal_transversal_plane);

		// right hand distance in coronal plane
		float coronal_distance_right_hand = math::dotProduct(right_hand_spine, normal_coronal_plane);

		// write distances into vector
		//pose.RightHandPosition.x = sagittal_distance_right_hand;
		//pose.RightHandPosition.y = transverse_distance_right_hand;

		pose.RightHandPosition.x = skeleton.SkeletonPositions[10].x;
		pose.RightHandPosition.y = skeleton.SkeletonPositions[10].y;
		pose.RightHandPosition.z = coronal_distance_right_hand;


		// left hand distance in sagittal plane 
		// 1. get vector from hip center to left wrist (we don't use shoulder center because it can be occluded)
		math::Vec3f left_hand_spine = Vector4ToVec3f(skeleton.SkeletonPositions[6]) - Vector4ToVec3f(skeleton.SkeletonPositions[1]);
		// 2. to get distance we project vector onto normal of saggital plane (we use lower saggital plane because hand can occlude shoulder)
		float sagittal_distance_left_hand = math::dotProduct(left_hand_spine, normal_sagittal_plane);

		// right hand distance in transverse plane
		float transverse_distance_left_hand = math::dotProduct(left_hand_spine, normal_transversal_plane);

		// right hand distance in coronal plane
		float coronal_distance_left_hand = math::dotProduct(left_hand_spine, normal_coronal_plane);

		// write distances into vector
		//pose.LeftHandPosition.x = sagittal_distance_left_hand;
		//pose.LeftHandPosition.y = transverse_distance_left_hand;
		pose.LeftHandPosition.x = skeleton.SkeletonPositions[6].x;
		pose.LeftHandPosition.y = skeleton.SkeletonPositions[6].y;
		pose.LeftHandPosition.z = coronal_distance_left_hand;

		// temporarily angles
		float cos_temp, temp1, temp2;

		//temporarily vectors
		math::Vec3f v, n, e1, e2;

		//sign
		float sign = 1.0f;

		spine.normalize();

		//horizontal adduction of left upper arm (angle in transverse plane)
		//project upper arm vector onto plane with spine vector as normal and compare against e1 = shoulder x spine and e2 = spine x e1
		e1 = math::crossProduct(shoulder_left, spine);
		e1.normalize();
		v = upper_arm_left - (math::dotProduct(upper_arm_left,spine) * spine); 
		v.normalize();
		e2 = math::crossProduct(spine,e1);
		e2.normalize();
		temp1 = math::dotProduct(v,e2);
		temp2 = math::dotProduct(v,e1);
		if (temp2 > 0.0f) sign = -1.0f;
		clamp(temp1);
		pose.AngleHorizontalAdductionArmLeft = acosf(temp1)*sign*180/MATH_PIf;
		sign = 1.0f;

		//horizontal adduction of right upper arm (angle in transverse plane)
		//project upper arm vector onto plane with spine vector as normal and compare against e1 = spine x shoulder and e2 = e1 x spine
		e1 = math::crossProduct(spine, shoulder_right);
		e1.normalize();
		v = upper_arm_right - (math::dotProduct(upper_arm_right,spine) * spine); 
		v.normalize();
		e2 = math::crossProduct(e1, spine);
		e2.normalize();
		temp1 = math::dotProduct(v,e2);
		temp2 = math::dotProduct(v,e1);
		if (temp2 > 0.0f) sign = -1.0f;
		clamp(temp1);
		pose.AngleHorizontalAdductionArmRight = acosf(temp1)*sign*180/MATH_PIf;
		sign = 1.0f;

		//rotation angle of upper left arm
		//construct orthonormal system at left shoulder
		e1 = math::crossProduct(upper_arm_left, spine);
		e2 = math::crossProduct(upper_arm_left, e1);
		e2.normalize();
		//project forearm vector into e1-e2-plane
		n = upper_arm_left;
		n.normalize();
		v = forearm_left - math::dotProduct(forearm_left, n) * n;
		v.normalize();
		cos_temp = math::dotProduct(v,e2);
		pose.AngleRotationUpperArmLeft = acosf(cos_temp)*180/MATH_PIf;

		//rotation angle of upper right arm
		//construct orthonormal system at right shoulder
		e1 = math::crossProduct(spine, upper_arm_right);
		e2 = math::crossProduct(e1, upper_arm_right);
		e2.normalize();
		//project forearm vector into e1-e2-plane
		n = upper_arm_right;
		n.normalize();
		v = forearm_right - math::dotProduct(forearm_right, n) * n;
		v.normalize();
		cos_temp = math::dotProduct(v,e2);
		pose.AngleRotationUpperArmRight = acosf(cos_temp)*180/MATH_PIf;


	
		upper_arm_left.normalize();
		forearm_left.normalize();
		upper_arm_right.normalize();
		forearm_right.normalize();
	


		//elevation angle of left upper arm
		cos_temp  = math::dotProduct(upper_arm_left, spine);
		clamp(cos_temp);
		pose.AngleElevationArmLeft = acosf(cos_temp)*180/MATH_PIf;

		//elevation angle of right upper arm
		cos_temp  = math::dotProduct(spine, upper_arm_right);
		clamp(cos_temp);
		pose.AngleElevationArmRight = acosf(cos_temp)*180/MATH_PIf;

		//angle of left elbow	
		cos_temp = math::dotProduct(-upper_arm_left, forearm_left);
		clamp(cos_temp);
		pose.AngleElbowLeft = acosf(cos_temp)*180/MATH_PIf;
	
		//angle of right elbow 
		cos_temp = math::dotProduct(-upper_arm_right, forearm_right);
		clamp(cos_temp);
		pose.AngleElbowRight = acosf(cos_temp)*180/MATH_PIf;

		// we don't get robust values for upper arm rotation if elbow angle is too large, so we set it to zero
		if (pose.AngleElbowLeft > 150) pose.AngleRotationUpperArmLeft = 0;
		if (pose.AngleElbowRight > 150) pose.AngleRotationUpperArmRight = 0;
	
		// distance of hands
		math::Vec3f wrist_left_to_wrist_right = Vector4ToVec3f(skeleton.SkeletonPositions[10]) -  Vector4ToVec3f(skeleton.SkeletonPositions[6]);
		pose.DistanceHandToHand = wrist_left_to_wrist_right.getLength();

		// angle of hand to hand vector towards coronal plane
		v = wrist_left_to_wrist_right - math::dotProduct(wrist_left_to_wrist_right, normal_coronal_plane) * normal_coronal_plane;
		v.normalize();
		temp1 = math::dotProduct(v,normal_sagittal_plane); 
		temp2 = math::dotProduct(v,normal_transversal_plane);
		if (temp2 > 0.0f) sign = -1.0f;
		clamp(temp1);
		pose.AngleCoronalHandToHandVector = acosf(temp1)*sign*180/MATH_PIf;
		sign = 1.0f;

		// angle of hand to hand vector towards tranverse plane
		v = wrist_left_to_wrist_right - math::dotProduct(wrist_left_to_wrist_right, normal_transversal_plane) * normal_transversal_plane;
		v.normalize();
		temp1 = math::dotProduct(v,normal_sagittal_plane);
		temp2 = math::dotProduct(v,normal_coronal_plane);
		if (temp2 < 0.0f) sign = -1.0f;
		clamp(temp1);
		pose.AngleTransversalHandToHandVector = acosf(temp1)*sign*180/MATH_PIf;
		sign = 1.0f;
	
		// distance of hands centroid towards coronal plane 
		math::Vec3f center_hands = (Vector4ToVec3f(skeleton.SkeletonPositions[10]) +  Vector4ToVec3f(skeleton.SkeletonPositions[6]))/2.0f;
		v = center_hands - Vector4ToVec3f(skeleton.SkeletonPositions[1]);
		pose.DistanceCenterHandsToCoronalPlane = math::dotProduct(v, normal_coronal_plane);

		// show coordinate system of upper part of body
		//glPushMatrix();
		//glTranslatef(skeleton.SkeletonPositions[2].x, skeleton.SkeletonPositions[2].y, skeleton.SkeletonPositions[2].z);
		//glScalef(.3f,.3f,.3f);
		//glBegin(GL_LINES);
		//	glColor3f(1.0,0.0,0.0);
		//	glVertex3f(0.0,0.0,0.0);
		//	glVertex3f(normal_upper_saggital_plane.x, normal_upper_saggital_plane.y, normal_upper_saggital_plane.z);
		//	
		//	glColor3f(0.0,1.0,0.0);
		//	glVertex3f(0.0,0.0,0.0);
		//	glVertex3f(normal_upper_transverse_plane.x, normal_upper_transverse_plane.y, normal_upper_transverse_plane.z);
		//	
		//	glColor3f(0.0,0.0,1.0);
		//	glVertex3f(0.0,0.0,0.0);
		//	glVertex3f(normal_upper_coronal_plane.x, normal_upper_coronal_plane.y, normal_upper_coronal_plane.z);
		//glEnd();
	 //   glPopMatrix();

		// show coordinate system of lower part of body
		//glPushMatrix();
		//glTranslatef(skeleton.SkeletonPositions[0].x, skeleton.SkeletonPositions[0].y, skeleton.SkeletonPositions[0].z);
		////glScalef(.3f,.3f,.3f);
		//glBegin(GL_LINES);
		//	glColor3f(.5,0.0,0.0);
		//	glVertex3f(0.0,0.0,0.0);
		//	glVertex3f(normal_lower_saggital_plane.x, normal_lower_saggital_plane.y, normal_lower_saggital_plane.z);
		//	
		//	glColor3f(0.0,.5,0.0);
		//	glVertex3f(0.0,0.0,0.0);
		//	glVertex3f(normal_lower_transverse_plane.x, normal_lower_transverse_plane.y, normal_lower_transverse_plane.z);
		//	
		//	glColor3f(0.0,0.0,.5);
		//	glVertex3f(0.0,0.0,0.0);
		//	glVertex3f(normal_lower_coronal_plane.x, normal_lower_coronal_plane.y, normal_lower_coronal_plane.z);
		//glEnd();

	 //   glPopMatrix();
	}


	void setPose(Pose &pose, float ael, float aer, float aeal, float aear, float ahaal, float ahaar, float arupl, float arupr, float achhv, float athhv, float dhh, float dchcp, math::Vec3f lhp, math::Vec3f rhp)
	{
		pose.AngleElbowLeft = ael; 
		pose.AngleElbowRight = aer; 
		pose.AngleElevationArmLeft = aeal; 
		pose.AngleElevationArmRight = aear;
		pose.AngleHorizontalAdductionArmLeft = ahaal;
		pose.AngleHorizontalAdductionArmRight = ahaar;
		pose.AngleRotationUpperArmLeft = arupl;
		pose.AngleRotationUpperArmRight = arupr;
		pose.AngleCoronalHandToHandVector = achhv;
		pose.AngleTransversalHandToHandVector = athhv;
		pose.DistanceHandToHand = dhh;
		pose.DistanceCenterHandsToCoronalPlane = dchcp;
		pose.LeftHandPosition = lhp;
		pose.RightHandPosition = rhp;
	}

	
	// returns euclidean distance of two poses
	float getPoseDistance(Pose &pose1, Pose &pose2, GestureType side_of_body)
	{
		Pose temp = pose2 - pose1;
		switch(side_of_body)
		{
		case LEFT:
			return sqrtf(temp.getSquaredLengthOfLeftAngles());
			break;
		case RIGHT:
			return sqrtf(temp.getSquaredLengthOfRightAngles());
			break;
		case BOTH:
			//return 0.5 * ( sqrtf(temp.getSquaredLengthOfLeftAngles()) + sqrtf(temp.getSquaredLengthOfRightAngles()) );  // <-- should we really do that??
			return sqrtf(temp.getSquaredLengthOfAngles());
			break;
		default:
			return 1000000000.0f;
		}
	}

	
	// returns length of time serie =  sum of euclidean distances between single elements 
	float getPathLength(std::vector<Pose>& posevector)
	{
		float path_length = 0;
		for (unsigned int i = 1; i < posevector.size(); i++)
		{
			path_length += getPoseDistance(posevector[i-1], posevector[i]);
		}
		return path_length;
	}


	// returns average element-wise euclidean distance
	float getEuclideanGestureDistance(Gesture& gest1, Gesture& gest2, GestureType side_of_body)
	{	
		assert(gest1.getSize() == gest2.getSize());
		float distance = 0;
		for (unsigned  int i = 0; i < gest1.getSize(); i++)
		{
			distance += pow(getPoseDistance(gest1.getPose(i), gest2.getPose(i), side_of_body),2);
		}
		distance = sqrtf(distance);
		// we scale along the number of data sets
		distance /= (float) gest1.getSize();
		return distance;
	}


	// returns element-wise variances in a vector
	std::vector<float> getGestureVariances(Gesture& gesture)
	{
		int N = gesture.getSize();
		Pose sums_of_squared_values;
		Pose means;
		for (int i = 0; i < N; i++)
		{
			means += gesture.getPose(i);
			sums_of_squared_values += gesture.getPose(i) * gesture.getPose(i);
		}

		// calculate the means
		means *= 1.0f/((float) N); 

		// calculate variance
		Pose variances = (sums_of_squared_values * 1.0f/((float) N)) - (means * means);

		return getFloatVectorFromPose(variances);
	}


	void centerGesture(Gesture &gest)
	{
		Pose mean = getMeanPose(gest);

		for (unsigned  int i = 0; i < gest.getSize(); i++)
		{
			
			gest.getPose(i) = gest.getPose(i) - mean;
		}
	}	


	void standardizeGesture(Gesture &gest)
	{
		Pose mean = getMeanPose(gest);
	
		Pose std_deviation;
		for (unsigned  int i = 0; i < gest.getSize(); i++)
		{
			//center the data
			gest.getPose(i) = gest.getPose(i) - mean;
			// we calculate the variances from the mean free data
			std_deviation += gest.getPose(i) * gest.getPose(i);
		}

	
		std_deviation = GestureUtility::getSquareRoot(std_deviation * (1.0f/((float) gest.getSize())));
		for (unsigned int i = 0; i < gest.getSize(); i++)
		{
			gest.getPose(i) = gest.getPose(i) / std_deviation;
		}
	}	


	void zeroLeftSideOfBody(Gesture &gest)
	{
		for (unsigned  int i = 0; i < gest.getSize(); i++)
		{
			gest.getPose(i).AngleElbowLeft = 0;
			gest.getPose(i).AngleElevationArmLeft = 0;
			gest.getPose(i).AngleHorizontalAdductionArmLeft = 0;
			gest.getPose(i).AngleRotationUpperArmLeft = 0;
		}
	}


	void zeroRightSideOfBody(Gesture &gest)
	{
		for (unsigned  int i = 0; i < gest.getSize(); i++)
		{
			gest.getPose(i).AngleElbowRight = 0;
			gest.getPose(i).AngleElevationArmRight = 0;
			gest.getPose(i).AngleHorizontalAdductionArmRight = 0;
			gest.getPose(i).AngleRotationUpperArmRight = 0;
		}
	}

	
	Pose getMeanPose(Gesture &gest)
	{
		Pose mean;
		for (unsigned  int i = 0; i < gest.getSize(); i++)
		{
			mean += gest.getPose(i);
		}

		mean *= 1.0f/((float) gest.getSize()); 
		return mean;
	}

	
	void  resamplePoseVector(std::vector<Pose>& posevector, int nr_of_poses)
	{
		// do nothing if there is no pose
		if (posevector.size() == 0)
		{
			return;
		}
		
		std::vector<Pose> new_posevector;

		// if the pose vector has size 1 (in case of STATIC gestures)
		if (posevector.size() == 1)
		{
			for (int i = 0; i < nr_of_poses; i++)
			{
				// we put the same vector into the list again and again
				new_posevector.push_back(posevector.front());
			}
		}
		else
		{
			std::vector<Pose>::iterator it;

			// we divide the gesture into evenly spaced intervals
			float interval_length = getPathLength(posevector) / ((float) (nr_of_poses - 1) );
			float temp_dist = 0.0f;

			// we always take the first...
			new_posevector.push_back(posevector[0]);

			// ... all the inbetween...*
			for (unsigned int i = 1; i < posevector.size(); i++)
			{
				float dist = getPoseDistance(posevector[i-1], posevector[i]);
				if ( (temp_dist + dist) > interval_length )
				{
					Pose new_pose = posevector[i-1] + (posevector[i] - posevector[i-1]) * ((interval_length - temp_dist)/(dist));
					new_posevector.push_back(new_pose);
					it = posevector.begin();
					posevector.insert(it + i, new_pose); // <- this is our i in the next iteration step 
					temp_dist = 0.0f;
				}
				else
				{
					temp_dist += dist;
				}

				if(new_posevector.size() > 10000)
				{
					// emergency stop: if user is not standing in front of kinect or is sitting too closely we get a huge vector with equal entries
					// due to calculation inacurracy at zero distances inbetween to frames we reset the vector and exit
					Pose front = posevector.front();
					posevector.clear();
					for (int i = 0; i < nr_of_poses; i++)
					{
						// we put the same vector into the list again and again
						posevector.push_back(front);
						return;
					}
				}
			}
			// due to calculation inacurracy the size of the new vector doesn't match sometimes
			if (new_posevector.size() == nr_of_poses)
			{
				new_posevector.pop_back();
			}
			// *...and the last original element into our resampled pose 
			new_posevector.push_back(posevector[posevector.size()-1]);
		}

		// swap the newly created vector with the old one
		posevector.swap(new_posevector);
	}


	Pose getSquareRoot(Pose &pose)
	{
		return Pose(sqrtf(pose.AngleElbowLeft), 
					sqrtf(pose.AngleElbowRight), 
					sqrtf(pose.AngleElevationArmLeft), 
					sqrtf(pose.AngleElevationArmRight), 
					sqrtf(pose.AngleHorizontalAdductionArmLeft),
					sqrtf(pose.AngleHorizontalAdductionArmRight),
					sqrtf(pose.AngleRotationUpperArmLeft),
					sqrtf(pose.AngleRotationUpperArmRight),
					sqrtf(pose.AngleCoronalHandToHandVector),
					sqrtf(pose.AngleTransversalHandToHandVector),
					sqrtf(pose.DistanceHandToHand),
					sqrtf(pose.DistanceCenterHandsToCoronalPlane),
					math::Vec3f(sqrtf(pose.LeftHandPosition.x), sqrtf(pose.LeftHandPosition.y),sqrtf(pose.LeftHandPosition.z)),
					math::Vec3f(sqrtf(pose.RightHandPosition.x), sqrtf(pose.RightHandPosition.y),sqrtf(pose.RightHandPosition.z)));
	}

	
	const char* GestureTypeToString(GestureType& type)
	{
		switch (type)
		{
		case LEFT:
			return "LEFT";
		case RIGHT:
			return "RIGHT";
		case BOTH:
			return "BOTH";
		case UNDEFINED:
			return "UNDEFINED";
		default:
			return "NO GESTURE TYPE";
		}
	}


	Pose operator * (const Pose &lhs, const Pose &rhs)
	{
		return Pose(lhs.AngleElbowLeft * rhs.AngleElbowLeft, 
					lhs.AngleElbowRight * rhs.AngleElbowRight, 
					lhs.AngleElevationArmLeft * rhs.AngleElevationArmLeft, 
					lhs.AngleElevationArmRight * rhs.AngleElevationArmRight, 
					lhs.AngleHorizontalAdductionArmLeft * rhs.AngleHorizontalAdductionArmLeft,
					lhs.AngleHorizontalAdductionArmRight * rhs.AngleHorizontalAdductionArmRight,
					lhs.AngleRotationUpperArmLeft * rhs.AngleRotationUpperArmLeft,
					lhs.AngleRotationUpperArmRight * rhs.AngleRotationUpperArmRight,
					lhs.AngleCoronalHandToHandVector * rhs.AngleCoronalHandToHandVector,
					lhs.AngleTransversalHandToHandVector * rhs.AngleTransversalHandToHandVector,
					lhs.DistanceHandToHand * rhs.DistanceHandToHand,
					lhs.DistanceCenterHandsToCoronalPlane * rhs.DistanceCenterHandsToCoronalPlane,
					math::Vec3f(lhs.LeftHandPosition.x * rhs.LeftHandPosition.x, lhs.LeftHandPosition.y * rhs.LeftHandPosition.y, lhs.LeftHandPosition.z * rhs.LeftHandPosition.z),
					math::Vec3f(lhs.RightHandPosition.x * rhs.RightHandPosition.x, lhs.RightHandPosition.y * rhs.RightHandPosition.y, lhs.RightHandPosition.z * rhs.RightHandPosition.z));
	}


	Pose operator * (const Pose &lhs, const float &rhs)
	{
		return Pose(lhs.AngleElbowLeft * rhs, 
					lhs.AngleElbowRight * rhs, 
					lhs.AngleElevationArmLeft * rhs, 
					lhs.AngleElevationArmRight * rhs, 
					lhs.AngleHorizontalAdductionArmLeft * rhs,
					lhs.AngleHorizontalAdductionArmRight * rhs,
					lhs.AngleRotationUpperArmLeft * rhs,
					lhs.AngleRotationUpperArmRight * rhs,
					lhs.AngleCoronalHandToHandVector * rhs,
					lhs.AngleTransversalHandToHandVector * rhs,
					lhs.DistanceHandToHand * rhs,
					lhs.DistanceCenterHandsToCoronalPlane * rhs,
					lhs.LeftHandPosition * rhs,
					lhs.RightHandPosition * rhs);
	}

	
	Pose operator / (const Pose &lhs, const Pose &rhs)
	{
		return Pose(lhs.AngleElbowLeft / rhs.AngleElbowLeft, 
					lhs.AngleElbowRight / rhs.AngleElbowRight, 
					lhs.AngleElevationArmLeft / rhs.AngleElevationArmLeft, 
					lhs.AngleElevationArmRight / rhs.AngleElevationArmRight, 
					lhs.AngleHorizontalAdductionArmLeft / rhs.AngleHorizontalAdductionArmLeft,
					lhs.AngleHorizontalAdductionArmRight / rhs.AngleHorizontalAdductionArmRight,
					lhs.AngleRotationUpperArmLeft / rhs.AngleRotationUpperArmLeft,
					lhs.AngleRotationUpperArmRight / rhs.AngleRotationUpperArmRight,
					lhs.AngleCoronalHandToHandVector / rhs.AngleCoronalHandToHandVector,
					lhs.AngleTransversalHandToHandVector / rhs.AngleTransversalHandToHandVector,
					lhs.DistanceHandToHand / rhs.DistanceHandToHand,
					lhs.DistanceCenterHandsToCoronalPlane / rhs.DistanceCenterHandsToCoronalPlane,
					math::Vec3f(lhs.LeftHandPosition.x / rhs.LeftHandPosition.x, lhs.LeftHandPosition.y / rhs.LeftHandPosition.y, lhs.LeftHandPosition.z / rhs.LeftHandPosition.z),
					math::Vec3f(lhs.RightHandPosition.x / rhs.RightHandPosition.x, lhs.RightHandPosition.y / rhs.RightHandPosition.y, lhs.RightHandPosition.z / rhs.RightHandPosition.z));
	}

	
	Pose operator / (const Pose &lhs, const float &rhs)
	{
		return Pose(lhs.AngleElbowLeft / rhs, 
					lhs.AngleElbowRight / rhs, 
					lhs.AngleElevationArmLeft / rhs, 
					lhs.AngleElevationArmRight / rhs, 
					lhs.AngleHorizontalAdductionArmLeft / rhs,
					lhs.AngleHorizontalAdductionArmRight / rhs,
					lhs.AngleRotationUpperArmLeft / rhs,
					lhs.AngleRotationUpperArmRight / rhs,
					lhs.AngleCoronalHandToHandVector / rhs,
					lhs.AngleTransversalHandToHandVector / rhs,
					lhs.DistanceHandToHand / rhs,
					lhs.DistanceCenterHandsToCoronalPlane / rhs,
					lhs.LeftHandPosition / rhs,
					lhs.RightHandPosition / rhs);
	}

	
	Pose operator + (const Pose &lhs, const Pose &rhs)
	{
		return Pose(lhs.AngleElbowLeft + rhs.AngleElbowLeft, 
			lhs.AngleElbowRight + rhs.AngleElbowRight, 
			lhs.AngleElevationArmLeft + rhs.AngleElevationArmLeft, 
			lhs.AngleElevationArmRight +  rhs.AngleElevationArmRight, 
			lhs.AngleHorizontalAdductionArmLeft + rhs.AngleHorizontalAdductionArmLeft,
			lhs.AngleHorizontalAdductionArmRight + rhs.AngleHorizontalAdductionArmRight,
			lhs.AngleRotationUpperArmLeft + rhs.AngleRotationUpperArmLeft,
			lhs.AngleRotationUpperArmRight + rhs.AngleRotationUpperArmRight,
			lhs.AngleCoronalHandToHandVector + rhs.AngleCoronalHandToHandVector,
			lhs.AngleTransversalHandToHandVector + rhs.AngleTransversalHandToHandVector, 
			lhs.DistanceHandToHand + rhs.DistanceHandToHand,
			lhs.DistanceCenterHandsToCoronalPlane + rhs.DistanceCenterHandsToCoronalPlane,
			lhs.LeftHandPosition + rhs.LeftHandPosition,
			lhs.RightHandPosition + rhs.RightHandPosition);
	}


	Pose operator - (const Pose &lhs, const Pose &rhs)
	{
		return Pose(lhs.AngleElbowLeft - rhs.AngleElbowLeft, 
			lhs.AngleElbowRight - rhs.AngleElbowRight, 
			lhs.AngleElevationArmLeft - rhs.AngleElevationArmLeft, 
			lhs.AngleElevationArmRight -  rhs.AngleElevationArmRight, 
			lhs.AngleHorizontalAdductionArmLeft - rhs.AngleHorizontalAdductionArmLeft,
			lhs.AngleHorizontalAdductionArmRight - rhs.AngleHorizontalAdductionArmRight,
			lhs.AngleRotationUpperArmLeft - rhs.AngleRotationUpperArmLeft,
			lhs.AngleRotationUpperArmRight - rhs.AngleRotationUpperArmRight,
			lhs.AngleCoronalHandToHandVector - rhs.AngleCoronalHandToHandVector,
			lhs.AngleTransversalHandToHandVector - rhs.AngleTransversalHandToHandVector, 
			lhs.DistanceHandToHand - rhs.DistanceHandToHand,
			lhs.DistanceCenterHandsToCoronalPlane - rhs.DistanceCenterHandsToCoronalPlane,
			lhs.LeftHandPosition - rhs.LeftHandPosition,
			lhs.RightHandPosition - rhs.RightHandPosition);
	}


	// return important angular values as vector
	std::vector<float> getFloatVectorFromPose(Pose &pose)
	{
		std::vector<float> result;

		result.push_back(pose.AngleElbowLeft);
		result.push_back(pose.AngleElevationArmLeft);
		result.push_back(pose.AngleHorizontalAdductionArmLeft);
		result.push_back(pose.AngleRotationUpperArmLeft);
		
		result.push_back(pose.AngleElbowRight);
		result.push_back(pose.AngleElevationArmRight);
		result.push_back(pose.AngleHorizontalAdductionArmRight);
		result.push_back(pose.AngleRotationUpperArmRight);
		
		return result;
	}


	void writeGestureToDisk(Gesture* gesture, const char* filename, bool append)
	{
		std::ofstream file;

		if(append) 
		{
			file.open(filename, std::ios::out | std::ios::app);
		}
		else 
		{
			file.open(filename, std::ios::out | std::ios::trunc);
		}
		
		if (file.is_open())
		{
			for (unsigned int i = 0; i < gesture->getSize(); i++)
			{
				std::vector<float> temp_pose = getFloatVectorFromPose(gesture->getPose(i));
				for (unsigned int j = 0; j < temp_pose.size(); j++)
				{
					file<<temp_pose[j]<<"\t";
				}

				file<<"\n";
			}
		
		file.close();
		}
		else std::cout << "Unable to open file";
	}
}