/* 
Gesture Utility provides basic operations on poses and gestures.
*/
#pragma once

#include <fstream>
#include "Gesture.h"
#include "Pose.h"


namespace GestureUtility
{
	// sets pose from given parameters
	void  setPose(Pose &pose, float ael, float aer, float aeal, float aear, float ahaal, float ahaar, float arupl, float arupr, float achhv, float athhv, float dhh, float dchcp, math::Vec3f lhp, math::Vec3f rhp);
	// calculates pose entries from a kinect skeleton
	void  setPoseFromSkeletonData(NUI_SKELETON_DATA &skeleton, Pose &pose);
	// returns important angular entries from a pose as a std::vector
	std::vector<float> getFloatVectorFromPose(Pose &pose);
	// sets all the left body side entries to zero
	void zeroLeftSideOfBody(Gesture &gest);
	// sets all the right body side entries to zero
	void zeroRightSideOfBody(Gesture &gest);

	// distance between two poses
	float getPoseDistance(Pose& pose1, Pose& pose2, GestureType side_of_body=BOTH);
	// euclidean distance bewtween two gestures
	float getEuclideanGestureDistance(Gesture& gest1, Gesture& gest2, GestureType side_of_body=BOTH);
	
	// resamples a list of poses to a new size
	void  resamplePoseVector(std::vector<Pose>& posevector, int nr_of_poses);
	// return the accumulated pose distances whithin a gesture
	float getPathLength(std::vector<Pose>& posevector);
	
	// centers the given gesture
	void centerGesture(Gesture &gest);
	// studentization of given gesture
	void standardizeGesture(Gesture &gest);
	// returns the gesture's variance as a vector
	std::vector<float> getGestureVariances(Gesture& gesture);	
	// get the mean of gesture
	Pose getMeanPose(Gesture &gest);
	//returns the element-wise square root
	Pose getSquareRoot(Pose &pose);

	// write gesture data to a text file
	void writeGestureToDisk(Gesture* gesture, const char* filename, bool append);

	// returns a string representation of gesture type
	const char* GestureTypeToString(GestureType& type);


	//multiplication with scalar
	Pose operator * (const Pose &lhs, const float &rhs);
	//element-wise multiplication
	Pose operator * (const Pose &lhs, const Pose &rhs);
	//division with scalar
	Pose operator / (const Pose &lhs, const float &rhs);
	//element-wise division
	Pose operator / (const Pose &lhs, const Pose &rhs);
	//addition of two poses
	Pose operator + (const Pose &lhs, const Pose &rhs);
	//subtraction
	Pose operator - (const Pose &lhs, const Pose &rhs);
}
