/*
Wrapper class for Kinect Sensor API

*/
#pragma once

#include <windows.h> //we have to include windows.h first. otherwise there are thousands of compiler errors when we include nuiapi.h
#include <NuiApi.h>


class KinectSensor
{
public:
	KinectSensor(void);
	~KinectSensor(void);

	// inits the kinect sensor
	void init();
	// fetches a new video frame from kinect sensor, updates the video data array and returns it 
	const BYTE* updateVideoFrameData();
	// fetches a new depth frame from kinect sensor, updates depth data array and returns it (should be called each time in the main program loop)
	const BYTE* updateDepthFrameData();
	// returns the current video frame
	const BYTE* getCurrentVideoFrame();
	// returns the current depth frame
	const BYTE* getCurrentDepthFrame();
	// returns the depth map in millimeters values
	const USHORT* getDepthMap();

	// fetches a new skeleton frame from kinect sensor, updates the current skeleton data and returns it (should be called each time in the main program loop)
	NUI_SKELETON_DATA& updateSkeletonData();
	// returns the current skeleton data
	NUI_SKELETON_DATA& getCurrentSkeletonData();
	// returns a dummy skeleton with all entries set to zero
	NUI_SKELETON_DATA getDummySkeleton();
	// sets the sensors tilt angle
	void setCameraAngle(LONG angle);
	// gets the sensors tilt angle
	LONG getCameraAngle();

private:

	// pointer to kinect sensor
	INuiSensor* NuiSensor;
	// handle to streams
	HANDLE VideoStreamHandle;
	HANDLE DepthStreamHandle;
	//active skeleton index
	int ActiveSkeletonID; 

	// current skeleton
	NUI_SKELETON_DATA CurrentSkeletonData;

	//buffer to video and depth frame data
	BYTE* VideoData;
	BYTE* DepthData;
	// depth map with millimeter values
	USHORT* DepthMap;
};

