#include "KinectTerrain.h"



// FOR DEBUGGING PURPOSES, use mouse as right hand input *********************************************
#ifdef MOUSEDEBUG 
	int LastMouseX = 0;
	int LastMouseY = 0;
	bool MousePressed = false;


	void KinectTerrain::PassiveMotion(int x, int y)
	{
		LastMouseX = x;
		LastMouseY = instance->WindowProps.WindowHeight - y;
	}

	void KinectTerrain::MouseFunc(int button, int state, int x, int y)
	{
		if( button == GLUT_LEFT_BUTTON )
		{
			if ( state == GLUT_DOWN ) 
			{
				MousePressed = true;
			}
			if ( state == GLUT_UP ) 
			{
				MousePressed = false;
			}
		}
	}
#endif
// ***************************************************************





KinectTerrain* KinectTerrain::instance = 0;


// GLUT CALLBACK WRAPPERS
void KinectTerrain::idleWrapper(){ instance->idle(); }
void KinectTerrain::updateWrapper(){ instance->update();}
void KinectTerrain::reshapeWindowWrapper(int width, int height){ instance->reshapeWindow(width,height);}
void KinectTerrain::handleKeyEventWrapper(unsigned char key, int x, int y){ instance->handleKeyEvent(key, x, y);}
void KinectTerrain::handleSpecialKeyEventWrapper(int key, int x, int y){ instance->handleSpecialKeyEvent(key,x,y);}


// event handling
void  KinectTerrain::reshapeWindow(int width, int height)
{
	int tx, ty, tw, th;
	glutSetWindow(MainWindowID);
	GLUI_Master.get_viewport_area( &tx, &ty, &tw, &th );
	WindowProps.WindowWidth = tw;
	WindowProps.WindowHeight = th;
	WindowProps.WindowAspect = ((float) WindowProps.WindowWidth) / ((float) WindowProps.WindowHeight);

	glViewport(0, 0, WindowProps.WindowWidth, WindowProps.WindowHeight);
	
	camera->update();
	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf( camera->getProjectionMatrix().ma );
	glMatrixMode(GL_MODELVIEW);

	// reset the button positions because their positions are declared absolute
	setButtonPositions();
}

// key events
void  KinectTerrain::handleKeyEvent(unsigned char key, int x, int y)
{

	std::stringstream ss;
	std::string file;

	switch (key)
	{
	// exit the program
	case 27:			//ESC
		exit(0);

	// numbers keys choose tools directly
	case '1':
		// camera tool
		ActiveTool = ToolList[1];
		break;
	case '2':
		// terra brush tool
		ActiveTool = ToolList[2];
		break;

	case '3':
		// smooth brush tool
		ActiveTool = ToolList[3];
		break;

	case '4':
		// randomize brush tool
		ActiveTool = ToolList[4];
		break;

	case '5':
		// bulldoze brush tool
		ActiveTool = ToolList[5];
		break;

	case '6':
		// dump brush tool
		ActiveTool = ToolList[6];
		break;

	case '7':
		// color brush tool
		ActiveTool = ToolList[7];
		break;

	case '8':
		// color picker
		ActiveTool = ToolList[8];
		break;

	//// DEBUGGING CAMERA CONTROLS
	//case 'q':
	//	static_cast<OrbitManipulator*> (instance->ToolList[1])->Azimuth += 7.0f;
	//	static_cast<OrbitManipulator*> (instance->ToolList[1])->updateCamera();
	//	break;
	//case 'w':
	//	static_cast<OrbitManipulator*> (instance->ToolList[1])->Azimuth -= 7.0f;
	//	static_cast<OrbitManipulator*> (instance->ToolList[1])->updateCamera();
	//	break;
	//case 'e':
	//	static_cast<OrbitManipulator*> (instance->ToolList[1])->Distance += 2.0f;
	//	static_cast<OrbitManipulator*> (instance->ToolList[1])->updateCamera();
	//	break;
	//case 'r':
	//	static_cast<OrbitManipulator*> (instance->ToolList[1])->Distance -= 2.0f;
	//	static_cast<OrbitManipulator*> (instance->ToolList[1])->updateCamera();
	//	break;
	////////

	case 'n':
		// show the surface normals as lines
		Mesh->toggleDisplaySurfaceNormals();
		break;

	case 's':
		// save maps
		Mesh->saveColorAndHeightMap();
		break;

	case 'l':
		// load gesture set file
		file = FileHandling::open("*.bin", WindowProps.WindowTitle);
		if(GRecognizer->loadTrainedGestures(file))
		{
			ss<<"Loaded "<<GRecognizer->getSizeOfGestureSets()<<" gesture sets from disk...";
			InfoText->setLineOfText(ss.str(), 30, true, 5);
			InfoText->activate();
		}
		break;

	case 'f':
		// let's go fullscreen
		glutFullScreenToggle();
		break;

	case 'd':
		//toggle depth data visualization
		RenderDepthFrame = !RenderDepthFrame;
		break;
	}
}


void  KinectTerrain::handleSpecialKeyEvent(int key, int x, int y)
{
	// (alt + ) up and down key to change kinect tilt
	int angle_inc = 0;
	
	if(key == GLUT_KEY_UP || key == GLUT_KEY_DOWN)
		angle_inc = (glutGetModifiers() == GLUT_ACTIVE_CTRL)?(10):(2);

	if(key == GLUT_KEY_DOWN)
		angle_inc *= -1;
	
	LONG angle = NuiSensor->getCameraAngle();
	NuiSensor->setCameraAngle(angle + angle_inc);	
}



void  KinectTerrain::handleHandEvent(std::string ev){} // actually no single hand events are used


// button events
void KinectTerrain::handleButtonEvent(std::string ev)
{
	if (ev == "BUTTON_ENTER_MENU")
	{
		for (unsigned int i = 1; i < ButtonList.size(); i++)
		{
			// activate all tool buttons
			ButtonList[i]->setActive();
		}
		// if menu button was activated we fade in the menu...
		ButtonMenuState = M_FADE_IN;
		// ...and deactivate the button
		ButtonList[0]->setInactive();
	}

	else
	{
		if (ev == "BUTTON_LEAVE_MENU")
		{	
		}

		if (ev == "BUTTON_CAMERA")
		{
			// camera tool
			ActiveTool = ToolList[1];
			HandleGestures = false;		
		}

		if(ev == "BUTTON_TERRA_BRUSH")
		{
			// terra brush tool
			ActiveTool = ToolList[2];
			HandleGestures = false;
		}

		if(ev == "BUTTON_SMOOTH_BRUSH")
		{
			// smooth brush tool
			ActiveTool = ToolList[3];
			HandleGestures = false;
		}

		if(ev == "BUTTON_RANDOMIZE_BRUSH")
		{
			// randomize brush tool
			ActiveTool = ToolList[4];
			HandleGestures = false;
		}

		if(ev == "BUTTON_BULLDOZE_BRUSH")
		{
			//  bulldoze brush tool
			ActiveTool = ToolList[5];
			HandleGestures = false;
		}

		if(ev == "BUTTON_DUMP_BRUSH")
		{
			//  dump brush tool
			ActiveTool = ToolList[6];
			HandleGestures = false;
		}

		if(ev == "BUTTON_COLOR_BRUSH")
		{
			//  color brush tool
			ActiveTool = ToolList[7];
			HandleGestures = false;
		}

		if(ev == "BUTTON_COLOR_PICKER")
		{
			//  color picker
			ActiveTool = ToolList[8];
			HandleGestures = false;
		}


		for (unsigned int i = 1; i < ButtonList.size(); i++)
		{
			// if tool button was pressed we deactivate the menu
			ButtonList[i]->setInactive();
		}
		// activate menu button...
		ButtonList[0]->setActive();
		// ... and fade out the menu
		ButtonMenuState = M_FADE_OUT;
	}
}


void  KinectTerrain::handleGestureEvent(std::string ev)
{
	// the default gesture (re-)activates the gesture recognition
	if(ev == "DEFAULT_GESTURE")
	{
		ActiveTool = ToolList[0];
		HandleGestures = true;
		// we return from here, hence we do not set a text message
		return;
	}

	// prepare text line for message
	std::stringstream ss;
	ss<<"Gesture "<<ev<<" recognized";
	InfoText->setLineOfText(ss.str(), 30, true, 5);


	if(ev == "0")
	{
		// camera tool
		ActiveTool = ToolList[1];
		HandleGestures = false;		
	}


	if(ev == "1")
	{
		// terra brush tool
		ActiveTool = ToolList[2];
		HandleGestures = false;
	}

	if(ev == "2")
	{
		// smooth brush tool
		ActiveTool = ToolList[3];
		HandleGestures = false;
	}

	if(ev == "3")
	{
		// randomize brush tool
		ActiveTool = ToolList[4];
		HandleGestures = false;
	}

	if(ev == "4")
	{
		// bulldoze brush tool
		ActiveTool = ToolList[5];
		HandleGestures = false;
	}

	if(ev == "5")
	{
		// dump brush tool
		ActiveTool = ToolList[6];
		HandleGestures = false;
	}

	if(ev == "6")
	{
		// color brush tool
		ActiveTool = ToolList[7];
		HandleGestures = false;
	}

	if(ev == "7")
	{
		// color picker
		ActiveTool = ToolList[8];
		HandleGestures = false;
	}

	// show text line about recognition result
	InfoText->activate();
}


KinectTerrain::KinectTerrain(void)
{
}


KinectTerrain::~KinectTerrain(void)
{
}


// inits program parameters and glut stuff
void KinectTerrain::init(int argc, char** argv)
{
	WindowProps.WindowAspect = 1.6f;
	WindowProps.WindowHeight = 1000;
	WindowProps.WindowWidth =  1600;
	WindowProps.WindowTitle = "*** KINECT TERRAIN MODELER ***";

	// there's a tool active by default, hence we let not recognize after the program start
	HandleGestures = false;
	
	//init GLUT & Window
	glutInit(&argc, argv);
	glutInitWindowPosition(0, 0);
	glutInitWindowSize(WindowProps.WindowWidth, WindowProps.WindowHeight);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
	MainWindowID = glutCreateWindow(WindowProps.WindowTitle);
	
	WindowProps.WindowWidth = glutGet(GLUT_WINDOW_WIDTH);
	WindowProps.WindowHeight = glutGet(GLUT_WINDOW_HEIGHT);
	WindowProps.WindowAspect = ((float) WindowProps.WindowWidth) / ((float) WindowProps.WindowHeight);

	// register callbacks
	glutDisplayFunc(updateWrapper);

#ifdef MOUSEDEBUG
	glutPassiveMotionFunc(PassiveMotion);
	glutMotionFunc(PassiveMotion);
	glutMouseFunc(MouseFunc);
#endif

	// Register the callbacks with GLUI (instead of GLUT registration) 
	GLUI_Master.set_glutIdleFunc(idleWrapper);
	GLUI_Master.set_glutReshapeFunc( reshapeWindowWrapper );
	GLUI_Master.set_glutKeyboardFunc( handleKeyEventWrapper );
	GLUI_Master.set_glutSpecialFunc( handleSpecialKeyEventWrapper );

	// set up kinect sensor and recognizers
	NuiSensor = new KinectSensor();
	GRecognizer = new GestureRecognizer(NuiSensor);
	LeftHandRecognizer = new HandRecognizer(NuiSensor, H_LEFT);
	RightHandRecognizer = new HandRecognizer(NuiSensor, H_RIGHT);
	LeftHandPointer = new KinectPointer(NuiSensor, P_LEFT, &WindowProps);
	LeftHandPointer->setPointerColor(0,0,255,255);
	RightHandPointer = new KinectPointer(NuiSensor, P_RIGHT, &WindowProps);
	RightHandPointer->setPointerColor(255,0,0,255);

	// load gesture sets from command line
	if (argc == 2)
	{
		// if there's one command line argument we interpret that as a filename
		GRecognizer->loadTrainedGestures(std::string(argv[1]));
	}

	// setup a new mesh
	Mesh = new GridMesh();

	// text renderer that shows recognition infos
	InfoText = new TextureTextRenderer();
	InfoText->setDuration(1500);
	InfoText->setPosition(0,0);
	InfoText->setTextColor(0,0,0,255);

	// null tool which does nothing
	Tool* nullTool = new NullTool();

	// camera and manipulator tool
	camera = new Camera(&WindowProps);
	Tool* cameraManipulator = new OrbitManipulator(camera);

	// color picker
	Tool* colorPicker = new Colorpicker(&WindowProps);

	// brush tool
	Tool* terra_brush = new TerraBrush(RightHandPointer, Mesh);

	// smooth tool
	Tool* smooth_brush = new SmoothBrush(RightHandPointer, Mesh);

	// randomize tool
	Tool* rand_brush = new RandomizeBrush(RightHandPointer, Mesh);

	// bulldoze tool
	Tool* bulldoze_brush = new BulldozeBrush(RightHandPointer, Mesh);

	// dump tool
	Tool* dump_brush = new DumpBrush(RightHandPointer, Mesh);

	// colorizer tool
	Tool* color_brush = new TerraColorizer(RightHandPointer, Mesh, &WindowProps);

	// let's put the tools into a list
	ToolList.push_back(nullTool);
	ToolList.push_back(cameraManipulator);
	ToolList.push_back(terra_brush);
	ToolList.push_back(smooth_brush);
	ToolList.push_back(rand_brush);
	ToolList.push_back(bulldoze_brush);
	ToolList.push_back(dump_brush);
	ToolList.push_back(color_brush);
	ToolList.push_back(colorPicker);
	
	// activate the terra brush as default
	ActiveTool = ToolList[2];

	// we provide a button based menu
	float radius = 64;
	CircleButton* menu_button = new CircleButton("BUTTON_ENTER_MENU", "../images/menu_button.png", 0, 0, radius*0.75f, 255, 255, 255); 
	menu_button->setActive();

	CircleButton* button1	  = new CircleButton("BUTTON_CAMERA", "../images/camera_3d.png", 0, 0, radius, 255, 255, 255); 
	CircleButton* button2     = new CircleButton("BUTTON_TERRA_BRUSH", "../images/terra_brush.png", 0, 0, radius, 255, 255, 255);
	CircleButton* button3     = new CircleButton("BUTTON_SMOOTH_BRUSH", "../images/smooth_brush.png", 0, 0, radius, 255, 255, 255);
	CircleButton* button4     = new CircleButton("BUTTON_RANDOMIZE_BRUSH", "../images/randomize_brush.png", 0, 0, radius, 255, 255, 255);
	CircleButton* button5     = new CircleButton("BUTTON_BULLDOZE_BRUSH", "../images/bulldoze_brush.png", 0, 0, radius, 255, 255, 255);
	CircleButton* button6     = new CircleButton("BUTTON_DUMP_BRUSH", "../images/dump_brush.png", 0, 0, radius, 255, 255, 255);
	CircleButton* button7     = new CircleButton("BUTTON_COLOR_BRUSH", "../images/brushtool.png", 0, 0, radius, 255, 255, 255);
	CircleButton* button8     = new CircleButton("BUTTON_COLOR_PICKER", "../images/colorpicker.png", 0, 0, radius, 255, 255, 255);
	CircleButton* exit_button = new CircleButton("BUTTON_LEAVE_MENU", "../images/exit_button.png", 0, 0, radius*0.75f, 255, 255, 255);

	// push buttons into a list
	ButtonList.push_back(menu_button);
	ButtonList.push_back(button1);
	ButtonList.push_back(button2);
	ButtonList.push_back(button3);
	ButtonList.push_back(button4);
	ButtonList.push_back(button5);
	ButtonList.push_back(button6);
	ButtonList.push_back(button7);
	ButtonList.push_back(button8);
	ButtonList.push_back(exit_button);

	// we arrange the buttons in a circular shape
	setButtonPositions();

	// init menu state
	ButtonMenuState = M_INACTIVE;
	AnimationCounter = 30;

	// dummy texture for Depth Frame
	glGenTextures(1, &DepthTextureID);
	int size = 1024;
	char * texturedata = new char[size*size*4];
	for (int i=0; i<size*size*4; i++)
	{
		texturedata[i] = 0;		
	}
	glBindTexture(GL_TEXTURE_2D, DepthTextureID);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, size, size, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, texturedata);

	delete[] texturedata;

	RenderDepthFrame = false;

	// init opengl stuff
	glEnable(GL_DEPTH_TEST);
	glViewport(0,0,WindowProps.WindowWidth,WindowProps.WindowHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf( camera->getProjectionMatrix().ma );
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	setStandardClearColor();
	glEnable (GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// Lighting
/*  <- place one more slash at the beginnung of the line to toggle comment blocks

	GLfloat mat_specular[] =  {0.0f, 0.0f, 0.0f};
	GLfloat mat_shininess[] = {0.0f};
	GLfloat mat_ambient[] =   {0.3f, 0.3f, 0.3f};
	GLfloat mat_diffuse[] =   {0.0f, 0.4f, 0.66f};

	GLfloat light_ambient[] = {0.0f, 0.0f, 0.0f};
	GLfloat light_diffuse[] = {1.0f, 1.0f, 1.0f};
	GLfloat light_speular[] = {0.0f, 0.0f, 0.0f};
	//GLfloat light_position[] =  {0.0f, 1.0f, 1.0f, 0.0f};

	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,  mat_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,   mat_ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,   mat_diffuse);
		
	glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR,  light_speular);
	//glLightfv(GL_LIGHT0, GL_POSITION, light_position);
		
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_LIGHT0); 

/*/
	glEnable(GL_COLOR_MATERIAL);  // standard settings are sufficient at the moment, for more customization check the material parameters
	glEnable(GL_LIGHT0);
//*/

	// load texture for frame that will hold the current color
	ColorFrameTextureID = SOIL_load_OGL_texture("../images/frame.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
}


void KinectTerrain::run()
{
	// invoke the glut loop
	glutMainLoop();
}


void KinectTerrain::idle()
{
	// redraw the scene
	glutPostRedisplay();
}

void KinectTerrain::update()
{
	//check if button events occurred
	while (CircleButton::isEventWaiting())
	{
		handleButtonEvent(CircleButton::getLastEvent());
	}

	//check if hand events occurred
	while (HandRecognizer::isEventWaiting())
	{
		handleHandEvent(HandRecognizer::getLastEvent());
	}

	//check if gesture events occurred
	while (GRecognizer->isEventWaiting())
	{
		handleGestureEvent(GRecognizer->getLastEvent());
	}


	// get new skeleton frame from kinect
	CurrentSkeleton = NuiSensor->updateSkeletonData();
	
	// update video and depth frames
	NuiSensor->updateVideoFrameData();
	NuiSensor->updateDepthFrameData();

	// show the video frame (e.g. in case of sensor angle adjustment)
	if(RenderDepthFrame)
	{
		showDepthFrame();
	}
	else
	// else continue the basic program loop
	{
		// recognize and update current pose
		if (HandleGestures)
		{
			CurrentPose = GRecognizer->update(CurrentSkeleton, true);
		}
		else
		{
			CurrentPose = GRecognizer->update(CurrentSkeleton, false);
		}
	
		// try to recognize hand inputs
		LeftHandRecognizer->update();
		RightHandRecognizer->update();
	
		// update pointers
		RightHandPointer->update(CurrentPose);
		LeftHandPointer->update(CurrentPose);

		//// FOR DEBUGGING PURPOSES
#ifdef MOUSEDEBUG
		RightHandPointer->setWindowCoordinates(LastMouseX, LastMouseY);
		////
#endif

		// handle tools only when the menu is closed
		if(ButtonMenuState == M_INACTIVE) 
			updateActiveTool(CurrentPose);

		// render all the graphic elements
		render();
	}
}

void KinectTerrain::render()
{
	// load the view matrix from our 3d camera
	glLoadMatrixf( (GLfloat *)camera->getViewMatrix().ma );

	// prepare color and depth buffer
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// render mesh
	renderMesh();

	// render tool
	ActiveTool->render();

	// render 2D overlays in screen space
	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix(); //save projection transformations
	glLoadIdentity();
	glOrtho(0.0, WindowProps.WindowWidth, 0.0, WindowProps.WindowHeight,-1.0,1.0);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix(); //save modelview transformations
	glLoadIdentity();

	// render button menu
	renderButtonMenu();

	// render Pointers
	bool left_hand_active = LeftHandPointer->drawPointer(LeftHandRecognizer->getHandState() == H_CLOSED);
	bool right_hand_active = RightHandPointer->drawPointer(RightHandRecognizer->getHandState() == H_CLOSED);

	// update Buttons, right hand is preferred
	if(right_hand_active)
	{
		math::Vec2f coords = RightHandPointer->getWindowCoordinates();
		updateButtons(ButtonList, coords);
	}
	else 
	{
		if(left_hand_active)
		{
			math::Vec2f coords = LeftHandPointer->getWindowCoordinates();
			updateButtons(ButtonList, coords);
		}
	}

	// FOR DEBUG PURPOSES
#ifdef MOUSEDEBUG
	updateButtons(ButtonList, RightHandPointer->getWindowCoordinates());
#endif
	// show tool on the right side as textured quad
	glPushMatrix();
	glTranslatef((float) (WindowProps.WindowWidth - 148), (float) (WindowProps.WindowHeight - 148), 0.0f);
	glColor4d(1.0, 1.0, 1.0, 1.0);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, ActiveTool->getTextureID());
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 1.0f);		glVertex3f(0.0f, 0.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);		glVertex3f(128.0f, 0.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);		glVertex3f(128.0f, 128.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);		glVertex3f(0.0f, 128.0f, 0.0f);
	glEnd();
	glPopMatrix();


	// show current color in frame as textured frame
	glPushMatrix();
	glTranslatef((float) (WindowProps.WindowWidth - 148), 20.0f, 0.0f);
	glColor3fv(Colorpicker::CurrentColor.v);
	glBindTexture(GL_TEXTURE_2D, ColorFrameTextureID);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 1.0f);		glVertex3f(0.0f, 0.0f, 0.0f);
		glTexCoord2f(1.0f, 1.0f);		glVertex3f(128.0f, 0.0f, 0.0f);
		glTexCoord2f(1.0f, 0.0f);		glVertex3f(128.0f, 128.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);		glVertex3f(0.0f, 128.0f, 0.0f);
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
	
	// render text
	InfoText->render();

	// reset projection and modelview matrix
	glEnable(GL_DEPTH_TEST);
	glPopMatrix(); //restore modelview transformations
	glMatrixMode(GL_PROJECTION);
	glPopMatrix(); //restore projection transformations
	glMatrixMode(GL_MODELVIEW);

	// swap buffers
	glutSwapBuffers();
}


void KinectTerrain::renderMesh()
{
	// draw!
	Mesh->render();

	// render coordinate system
	/*glLineWidth(4.0f);
	
	glBegin(GL_LINES);
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
		glVertex3f(1,0,0);

		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(0,1,0);

		glColor3f(0,0,1);
		glVertex3f(0,0,0);
		glVertex3f(0,0,1);
	glEnd();*/
}


void KinectTerrain::updateActiveTool(Pose& pose)
{
	// check out active sides of body
	bool left_active = (LeftHandRecognizer->getHandState() == H_CLOSED) && (LeftHandPointer->isActive());
	bool right_active = (RightHandRecognizer->getHandState() == H_CLOSED) && (RightHandPointer->isActive());
	
	// DEBUGGING
#ifdef MOUSEDEBUG
	right_active = MousePressed;
#endif
	///


	// update active tool in respect to hand states and pointers
	int code = left_active + 2 * right_active;
	
	switch(code)
	{
	case 1:
		ActiveTool->updateLeftHandInput(pose);
		break;
	case 2:
		ActiveTool->updateRightHandInput(pose);
		break;
	case 3:
		ActiveTool->updateBothHandInput(pose);
		break;
	default:
		ActiveTool->update(pose);
	}
}


void KinectTerrain::setStandardClearColor()
{
	glClearColor(1.0, 1.0, 1.0, 1.0);
}

void KinectTerrain::updateButtons(std::vector<CircleButton*>& list, math::Vec2f& coords)
{
	// update all the states of the buttons in the list
	for (unsigned int i = 0; i < list.size(); i++)
	{
		list[i]->updateButtonState(coords.x, coords.y);
	}
}



void KinectTerrain::renderButtonMenu()
{
	//std::cout<<"\r"<<ButtonMenuState;
	glPushMatrix();

	// we always draw the "enter menu" button
	ButtonList[0]->drawButton();

	// let's model the menu states
	switch(ButtonMenuState)
	{
	case M_INACTIVE:
		break;

	case M_FADE_IN:
		// we do a fancy "fade in" animation...
		glTranslatef(0.5f * WindowProps.WindowWidth, 0.5f * WindowProps.WindowHeight, 0.0f);    // translate coordinate system to middle of the screen
		glRotatef(4.0f * AnimationCounter, 0.0f, 0.0f, 1.0f);									// rotate and scale
		glScalef(0.2f * AnimationCounter, 0.2f * AnimationCounter, 0.2f * AnimationCounter);
		glTranslatef(-0.5f * WindowProps.WindowWidth, -0.5f * WindowProps.WindowHeight, 0.0f);	// translate back

		//...and draw
		for (unsigned int i = 1; i < ButtonList.size()-1; i++)
		{
			ButtonList[i]->drawButton();
		}
		AnimationCounter--; // counter declares length of animation transition
		if(AnimationCounter == 5) // animation end is at counter==5 because we scale with factor 0.2
			ButtonMenuState = M_ACTIVE;
		break;

	case M_ACTIVE:
		// draw the buttons
		for (unsigned int i = 1; i < ButtonList.size(); i++)
		{
			ButtonList[i]->drawButton();
		}
		break;

	case M_FADE_OUT:
		// we do a fancy fade out animation...
		glTranslatef(0.5f * WindowProps.WindowWidth, 0.5f * WindowProps.WindowHeight, 0.0f);   // translate coordinate system to middle of the screen
		glRotatef(4.0f * AnimationCounter, 0.0f, 0.0f, 1.0f);								   // rotate and scale
		glScalef(0.2f * AnimationCounter, 0.2f * AnimationCounter, 0.2f * AnimationCounter);
		glTranslatef(-0.5f * WindowProps.WindowWidth, -0.5f * WindowProps.WindowHeight, 0.0f); // translate back
		//...and draw
		for (unsigned int i = 1; i < ButtonList.size()-1; i++)
		{
			ButtonList[i]->drawButton();
		}
		AnimationCounter++;
		if(AnimationCounter == 30) ButtonMenuState = M_INACTIVE;
		break;
	}

	glPopMatrix();
}

void KinectTerrain::setButtonPositions()
{
	float radius = ButtonList[0]->getRadius();
	// set the position of the menu button at the top right of the window (just below the active tool)
	ButtonList[0]->setPosition((float) (WindowProps.WindowWidth - 76), (float) (WindowProps.WindowHeight - 148) - 1.5f * radius);

	// all the other buttons are ordered in a circular shape
	float nr_of_buttons = (float) ButtonList.size()-2;
	for (unsigned int i=1; i < ButtonList.size()-1; i++)
	{
		float angle = 6.28319f / nr_of_buttons * (i - 1);
		ButtonList[i]->setPosition(200.0f * cos(angle) + 0.5f * WindowProps.WindowWidth, 200.0f * sin(angle) + 0.5f * WindowProps.WindowHeight);
	}

	// leave button in the center of the circle
	ButtonList[ButtonList.size()-1]->setPosition(0.5f * WindowProps.WindowWidth, 0.5f * WindowProps.WindowHeight);
}

void KinectTerrain::showDepthFrame()
{
	glBindTexture(GL_TEXTURE_2D, DepthTextureID);
	// write depth data
	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 640, 480, GL_BGRA_EXT, GL_UNSIGNED_BYTE, NuiSensor->getCurrentDepthFrame());

	// write skeleton positions into depth map
	NUI_SKELETON_DATA skeleton = NuiSensor->getCurrentSkeletonData();

	BYTE marker[] = { 255, 255, 255, 255, 255,
					  255, 255, 255, 255, 255,
					  255, 255, 255, 255, 255,
					  255, 255, 255, 255, 255, 
					  255, 255, 255, 255, 255
					};
	for (int i=0; i<20; i++)
	{
		long x_coord = 0, y_coord = 0;
		unsigned short depth;
		NuiTransformSkeletonToDepthImage(skeleton.SkeletonPositions[i], &x_coord, &y_coord, &depth, NUI_IMAGE_RESOLUTION_640x480);

		glTexSubImage2D(GL_TEXTURE_2D, 0, x_coord-2, y_coord-2, 4, 4, GL_BLUE, GL_UNSIGNED_BYTE, marker);
	}	

	glPushMatrix(); //save modelview transformations
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glPushMatrix(); //save projection transformations
	glLoadIdentity();
	glOrtho(0.0,1.0,0.0,1.0,-1.0,1.0);

	glColor3f(1.0f,1.0f,1.0f);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, DepthTextureID);

	// calculate the extends of the area (of the 1024x1024 texture) where we can find the data (640x480)
	float coordx = 640.0f/1024.0f;
	float coordy = 480.0f/1024.0f;

	// set width of textured quad
	float width = 1.333f/WindowProps.WindowAspect;
	float margin = (1.0f - width) / 2.0f;

	// render
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, coordy);			glVertex3f(margin, 0.0f, 0.0f);
		glTexCoord2f(coordx, coordy);		glVertex3f(1.0f - margin, 0.0f, 0.0f);
		glTexCoord2f(coordx, 0.0f);			glVertex3f(1.0f - margin, 1.0f, 0.0f);
		glTexCoord2f(0.0f, 0.0f);			glVertex3f(margin, 1.0f, 0.0f);
	glEnd();

	glDisable(GL_TEXTURE_2D);

	glPopMatrix(); //restore projection transformations
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix(); //restore modelview transformations


	// swap buffers
	glutSwapBuffers();
}