#include "Pose.h"
#include "GestureUtility.h"

Pose::Pose(void)
{
	math::Vec3f lhp(0.0f, 0.0f, 0.0f);
	math::Vec3f rhp(0.0f, 0.0f, 0.0f);
	GestureUtility::setPose( *this, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, lhp, rhp);
}


Pose::Pose(float ael, float aer, float aeal, float aear, float ahaal, float ahaar, float arupl, float arupr, float achhv, float athhv, float dhh, float dchcp, math::Vec3f lhp, math::Vec3f rhp)
{
	GestureUtility::setPose( *this, ael,  aer,  aeal,  aear,  ahaal,  ahaar,  arupl,  arupr,  achhv,  athhv,  dhh,  dchcp, lhp, rhp);
}


Pose::~Pose(void)
{
}
	



void Pose::operator *= (const float &rhs)
{
	AngleElbowLeft *= rhs; 
	AngleElbowRight *= rhs; 
	AngleElevationArmLeft *= rhs; 
	AngleElevationArmRight *= rhs;
	AngleHorizontalAdductionArmLeft *= rhs;
	AngleHorizontalAdductionArmRight *= rhs;
	AngleRotationUpperArmLeft *= rhs;
	AngleRotationUpperArmRight *= rhs;
	AngleCoronalHandToHandVector *= rhs;
	AngleTransversalHandToHandVector *= rhs;
	DistanceHandToHand *= rhs;
	DistanceCenterHandsToCoronalPlane *= rhs;
	LeftHandPosition *= rhs;
	RightHandPosition *= rhs;
}


void Pose::operator += (const Pose &rhs)
{
	AngleElbowLeft += rhs.AngleElbowLeft; 
	AngleElbowRight += rhs.AngleElbowRight; 
	AngleElevationArmLeft += rhs.AngleElevationArmLeft;
	AngleElevationArmRight += rhs.AngleElevationArmRight;
	AngleHorizontalAdductionArmLeft += rhs.AngleHorizontalAdductionArmLeft;
	AngleHorizontalAdductionArmRight += rhs.AngleHorizontalAdductionArmRight;
	AngleRotationUpperArmLeft += rhs.AngleRotationUpperArmLeft;
	AngleRotationUpperArmRight += rhs.AngleRotationUpperArmRight;
	AngleCoronalHandToHandVector += rhs.AngleCoronalHandToHandVector;
	AngleTransversalHandToHandVector += rhs.AngleTransversalHandToHandVector;
	DistanceHandToHand += rhs.DistanceHandToHand;
	DistanceCenterHandsToCoronalPlane += rhs.DistanceCenterHandsToCoronalPlane;
	LeftHandPosition += rhs.LeftHandPosition;
	RightHandPosition += rhs.RightHandPosition;
}


void Pose::operator -= (const Pose &rhs)
{
	AngleElbowLeft -= rhs.AngleElbowLeft; 
	AngleElbowRight -= rhs.AngleElbowRight; 
	AngleElevationArmLeft -= rhs.AngleElevationArmLeft;
	AngleElevationArmRight -= rhs.AngleElevationArmRight;
	AngleHorizontalAdductionArmLeft -= rhs.AngleHorizontalAdductionArmLeft;
	AngleHorizontalAdductionArmRight -= rhs.AngleHorizontalAdductionArmRight;
	AngleRotationUpperArmLeft -= rhs.AngleRotationUpperArmLeft;
	AngleRotationUpperArmRight -= rhs.AngleRotationUpperArmRight;
	AngleCoronalHandToHandVector -= rhs.AngleCoronalHandToHandVector;
	AngleTransversalHandToHandVector -= rhs.AngleTransversalHandToHandVector;
	DistanceHandToHand -= rhs.DistanceHandToHand;
	DistanceCenterHandsToCoronalPlane -= rhs.DistanceCenterHandsToCoronalPlane;
	LeftHandPosition -= rhs.LeftHandPosition;
	RightHandPosition -= rhs.RightHandPosition;
}
	

void Pose::savePose(std::ofstream &open_file)
{
	// save all the poses
	open_file.write( (char*) &(AngleElbowLeft), sizeof(float));
	open_file.write( (char*) &(AngleElbowRight), sizeof(float));   
	open_file.write( (char*) &(AngleElevationArmLeft), sizeof(float));   
	open_file.write( (char*) &(AngleElevationArmRight), sizeof(float));  
	open_file.write( (char*) &(AngleHorizontalAdductionArmLeft), sizeof(float));  
	open_file.write( (char*) &(AngleHorizontalAdductionArmRight), sizeof(float));  
	open_file.write( (char*) &(AngleRotationUpperArmLeft), sizeof(float));  
	open_file.write( (char*) &(AngleRotationUpperArmRight), sizeof(float));  
	open_file.write( (char*) &(AngleCoronalHandToHandVector), sizeof(float));  
	open_file.write( (char*) &(AngleTransversalHandToHandVector), sizeof(float));  
	open_file.write( (char*) &(DistanceHandToHand), sizeof(float));  
	open_file.write( (char*) &(DistanceCenterHandsToCoronalPlane), sizeof(float));  
	open_file.write( (char*) &(LeftHandPosition.x), sizeof(float));  
	open_file.write( (char*) &(LeftHandPosition.y), sizeof(float)); 
	open_file.write( (char*) &(LeftHandPosition.z), sizeof(float)); 
	open_file.write( (char*) &(RightHandPosition.x), sizeof(float));  
	open_file.write( (char*) &(RightHandPosition.y), sizeof(float)); 
	open_file.write( (char*) &(RightHandPosition.z), sizeof(float)); 				
}

void Pose::loadPose(std::ifstream &open_file)
{
	open_file.read( (char*) &(AngleElbowLeft), sizeof(float));
	open_file.read( (char*) &(AngleElbowRight), sizeof(float));   
	open_file.read( (char*) &(AngleElevationArmLeft), sizeof(float));   
	open_file.read( (char*) &(AngleElevationArmRight), sizeof(float));  
	open_file.read( (char*) &(AngleHorizontalAdductionArmLeft), sizeof(float));  
	open_file.read( (char*) &(AngleHorizontalAdductionArmRight), sizeof(float));  
	open_file.read( (char*) &(AngleRotationUpperArmLeft), sizeof(float));  
	open_file.read( (char*) &(AngleRotationUpperArmRight), sizeof(float));  
	open_file.read( (char*) &(AngleCoronalHandToHandVector), sizeof(float));  
	open_file.read( (char*) &(AngleTransversalHandToHandVector), sizeof(float));  
	open_file.read( (char*) &(DistanceHandToHand), sizeof(float));  
	open_file.read( (char*) &(DistanceCenterHandsToCoronalPlane), sizeof(float));  
	open_file.read( (char*) &(LeftHandPosition.x), sizeof(float));  
	open_file.read( (char*) &(LeftHandPosition.y), sizeof(float)); 
	open_file.read( (char*) &(LeftHandPosition.z), sizeof(float)); 
	open_file.read( (char*) &(RightHandPosition.x), sizeof(float));  
	open_file.read( (char*) &(RightHandPosition.y), sizeof(float)); 
	open_file.read( (char*) &(RightHandPosition.z), sizeof(float));
}