/*
Class provides angular model that is calculated from skeleton data.
*/
#pragma once


#include "math\Vec3f.h"
#include <fstream>

class Pose
{
public:

	Pose(void);
	Pose(float ael, float aer, float aeal, float aear, float ahaal, float ahaar, float arupl, float arupr, float achhv, float athhv, float dhh, float dchcp, math::Vec3f lhp, math::Vec3f rhp);
	~Pose(void);

	// returns squared sum of important angular values (the 8 angles that are used in recognition system)
	float getSquaredLengthOfAngles();
	// returns squared sum of important angular values, only for left side of body
	float getSquaredLengthOfLeftAngles();
	// returns squared sum of important angular values, only for right side of body
	float getSquaredLengthOfRightAngles();

	// scalar multiplication
	void  operator *= (const float &rhs);
	// vector addition of poses
	void  operator += (const Pose &rhs);
	// vector subtraction of poses
	void  operator -= (const Pose &rhs);

	// file handling
	void  savePose(std::ofstream &open_file);
	void  loadPose(std::ifstream &open_file);



	//angles, the first 8 values are used for gesture recognition of upper body
	float AngleElbowLeft;
	float AngleElbowRight;
	float AngleElevationArmLeft;
	float AngleElevationArmRight;
	float AngleHorizontalAdductionArmLeft;
	float AngleHorizontalAdductionArmRight;
	float AngleRotationUpperArmLeft;
	float AngleRotationUpperArmRight;

	float AngleCoronalHandToHandVector;
	float AngleTransversalHandToHandVector;

	//distances
	float DistanceHandToHand;
	float DistanceCenterHandsToCoronalPlane;

	// hand positions (distances to sagittal/transverse/coronal plane)
	math::Vec3f LeftHandPosition;
	math::Vec3f RightHandPosition;

};

inline float Pose::getSquaredLengthOfAngles()
{
	return (getSquaredLengthOfLeftAngles() + getSquaredLengthOfRightAngles());
}

inline float Pose::getSquaredLengthOfLeftAngles()
{
	float result = 0.0f;
	result += AngleElbowLeft * AngleElbowLeft;
	result += AngleElevationArmLeft * AngleElevationArmLeft;
	result += AngleHorizontalAdductionArmLeft * AngleHorizontalAdductionArmLeft;
	result += AngleRotationUpperArmLeft * AngleRotationUpperArmLeft;
	return  result;
}

inline float Pose::getSquaredLengthOfRightAngles()
{
	float result = 0.0f;
	result  += AngleElbowRight * AngleElbowRight;
	result  += AngleElevationArmRight * AngleElevationArmRight;
	result  += AngleHorizontalAdductionArmRight * AngleHorizontalAdductionArmRight;
	result  += AngleRotationUpperArmRight * AngleRotationUpperArmRight;
	return  result;
}