/*
This class provides the texture-based rendering of one line of text.
We can set position, color and duration of text display
*/
#pragma once

#include <map>
#include <string>
#include <GL\freeglut.h>
#include <SOIL.h>
#include "TimeFunctions.h"

// here we collect all neccessary informations about the position of the letter in the texture file
struct LetterTexInfo
{
	char letter;
	float x_begin;
	float x_end;
	GLuint texID;
};

class TextureTextRenderer
{
public:
	
	TextureTextRenderer(void);
	~TextureTextRenderer(void);

	// set a line of text with given height and spacing. text can be displayed in a boxed frame
	void setLineOfText(const char* text, unsigned int text_height, bool boxed_text = 0, unsigned int letter_spacing = 0); 
	void setLineOfText(std::string text, unsigned int text_height, bool boxed_text = 0, unsigned int letter_spacing = 0); 
	// sets the duration and fade time (duration of -1 = infinite duration)
	void setDuration(int duration, unsigned int fade_time = 500);
	// sets the position of the text
	void setPosition(unsigned int x_position, unsigned int y_position);
	// sets the text color
	void setTextColor(unsigned char color_red, unsigned char color_green, unsigned char color_blue, unsigned char alpha);
	// sets up a new display list and start time
	void activate();
	// renders the text
	void render(); 

private:
	// text attributes
	std::string Text;
	unsigned int TextHeight; 
	unsigned int LetterSpacing;
	unsigned int XPosition;
	unsigned int YPosition;
	unsigned char RedChannel;
	unsigned char GreenChannal;
	unsigned char BlueChannel;
	unsigned char AlphaChannel;
	bool BoxedText;
	int TextDuration;
	unsigned int TextFadeDuration;
	clock_t StartTime;

	// texture file aspect
	static float TextureAspect;
	// texture IDs for font textures
	static GLuint CapitalsTextureID;
	static GLuint MinusculesTextureID;
	// a map to hold all texture information
	static std::map<char, LetterTexInfo> FontTexInformations;
	// do we already loaded the letter textures?
	static bool TexturesLoaded;
	
	// diplay list for lines of text
	GLuint TextDisplayList;
	GLuint BoxDisplayList;
	
	// helper function to insert a letter into the texture info list 
	void insertLetter(char letter, float x_begin, float x_end, GLuint texID);
	// set all the letter texture positions
	void setupLetters();
};

