/*
Tool to choose a global color used for brushes an ink tool. Based on HSV color space. 
Closed right hand controls Hue and Saturation. Closed left hand adjusts the color value.
*/
#pragma once

#include "Tool.h"
#include "math\Vec2f.h"
#include "math\Vec3f.h"
#include <GL/freeglut.h>
#include "WindowProperties.h"

class Colorpicker :	public Tool
{
public:
	Colorpicker(WindowProperties* window_props);
	~Colorpicker(void);

	void updateLeftHandInput(Pose &pose); 
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose); 

	void update(Pose &pose);
	void render();
	void undo(){}
	void redo(){}

	int getTextureID(){return TextureID;};
	// global color in RGB
	static math::Vec3f CurrentColor;

private:
	// clamps the hue to [0...360]
	void clampHue();
	// sets the values for Hue and Saturation (right hand action)
	void setHueAndSaturation(Pose& pose);
	// sets the Value (left hand action)
	void setValue(Pose& pose);
	// maintain pose from last iteration step
	Pose LastPose;
	
	// HSV color components
	float Value;
	float Hue;
	float Saturation;

	// pointer to window properties (used in render method)
	WindowProperties* WindowProps;
	// texture for tool icon
	int TextureID;
};

