/*
This tool provides filling. The grid points on the mesh are successively raised until a user-defined height is reached.
The closed right hand invokes the tool and sets its position. The closed left hand adjusts size and target height.
*/
#pragma once

#include "Tool.h"
#include "math\Vec3f.h"
#include "KinectPointer.h"
#include "KinectTerrain\GridMesh.h"

class DumpBrush: public Tool
{
public:
	DumpBrush(KinectPointer* right_hand, GridMesh* mesh);
	virtual ~DumpBrush(void);

	void updateLeftHandInput(Pose &pose);
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose);

	void update(Pose &pose);
	void render();
	void undo(){}
	void redo(){}

	// return texture id for editor icon
	int getTextureID(){return TextureID;}




private:
	// sets the tool parameters
	void setRadiusAndHeight(Pose& pose);
	// actual tool operation
	void processMesh();

	// pointer to hand pointer
	KinectPointer* RightHandPointer;
	// pointer to mesh
	GridMesh* Mesh;
	// maintain pose from last iteration step
	Pose LastPose;

	// brush parameters
	float BrushRadius;
	float BrushHeight;
	float Scale;
	// show brush if inside grid
	bool showBrush;
	// local coords of brush tip
	math::Vec3f BrushCoordinates;
	
	// tool icon texture
	int TextureID;
};

