#include "Ink2D.h"
#include "SOIL.h"
#include <iostream>

Ink2D::Ink2D(std::vector<Stroke>* strokes, int* selected_stroke_id)
{
	Strokes = strokes;
	SelectedStrokeID = selected_stroke_id;
	TextureID = SOIL_load_OGL_texture("../images/filltool.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
}
Ink2D::~Ink2D()
{
}

void Ink2D::updateLeftHandInput(Pose &pose)
{
}


void Ink2D::updateRightHandInput(Pose &pose)
{
	if(*SelectedStrokeID != -1)
	{
		// if we have a valid stroke id we try to get the corresponding stroke in the list
		for(unsigned int i = 0; i < Strokes->size(); i++)
		{
			if( (*Strokes)[i].ID == (*SelectedStrokeID) )
			{
				if( (*Strokes)[i].Color != Colorpicker::CurrentColor )
				{
					// save an undo information if neccessary
					if(UndoList.size() > 0)
					{
						// to avoid duplicates we only save the info if it's not the same as the last one
						InkInfo last_undo_element = UndoList.back();

						if( (*SelectedStrokeID) != last_undo_element.StrokeID )
						{
							InkInfo undo_element;
							undo_element.color = (*Strokes)[i].Color;
							undo_element.StrokeID = *SelectedStrokeID;
							UndoList.push_back(undo_element);
							// update the undo/redo manager
							UndoRedoManager::checkIn(this);
							// clear the redo list
							RedoList.clear();

							//std::cout<<*SelectedStrokeID<<" hit! Pushed back undo information!\n";
						}
					}
					else
					{
						InkInfo undo_element;
						undo_element.color = (*Strokes)[i].Color;
						undo_element.StrokeID = *SelectedStrokeID;
						UndoList.push_back(undo_element);
						// update the undo/redo manager
						UndoRedoManager::checkIn(this);
						// clear the redo list
						RedoList.clear();
					}
				}

				// set the new color
				(*Strokes)[i].Color = Colorpicker::CurrentColor;
			}
		}
	}
}	

void Ink2D::updateBothHandInput(Pose &pose)
{
	updateRightHandInput(pose);
}
	

void Ink2D::update(Pose &pose)
{
}	

void Ink2D::render()
{
}



void Ink2D::undo()
{
	if(UndoList.size() > 0)
	{
		//get the last element
		InkInfo last_element = UndoList.back();
		//pop it from the stack
		UndoList.pop_back();
		//try to get the corresponding stroke
		for(unsigned int i = 0; i < Strokes->size(); i++)
		{
			// if we found it...
			if( (*Strokes)[i].ID == last_element.StrokeID )
			{
				//... we save a redo information ...
				InkInfo redo_info;
				redo_info.StrokeID = last_element.StrokeID;
				redo_info.color = (*Strokes)[i].Color;
				RedoList.push_back(redo_info);
				
				//and set the original color
				(*Strokes)[i].Color = last_element.color;
			}
		}
	}
}

void Ink2D::redo()
{
	if(RedoList.size() > 0)
	{
		//get the last element
		InkInfo last_element = RedoList.back();
		//pop it from the stack
		RedoList.pop_back();
		//try to get the corresponding stroke
		for(unsigned int i = 0; i < Strokes->size(); i++)
		{
			// if we found it...
			if( (*Strokes)[i].ID == last_element.StrokeID )
			{
				//... we save an undo information ...
				InkInfo undo_info;
				undo_info.StrokeID = last_element.StrokeID;
				undo_info.color = (*Strokes)[i].Color;
				UndoList.push_back(undo_info);
				//and set the old color
				(*Strokes)[i].Color = last_element.color;
			}
		}
	}
}