#include "Manipulator2D.h"
#include "SOIL.h"

Manipulator2D::Manipulator2D(KinectPointer* left_hand, KinectPointer* right_hand)
{
	Rotation = 0.0f;
	RotationCenter = math::Vec2f(0.0f, 0.0f);
	Scale = 1.0f;
	Translation = math::Vec2f(0.0f, 0.0f);

	LastTransformationMatrix = math::Matrix44f::Identity();
	TransformationMatrix = math::Matrix44f::Identity();
	LeftHandPointer = left_hand;
	RightHandPointer = right_hand;

	LastRotationCenter = math::Vec2f(0.0f, 0.0f);
	LastTranslation = math::Vec2f(0.0f, 0.0f);
	LastRotation = 0.0f;
	LastScale = 1.0f;

	TextureID = SOIL_load_OGL_texture("../images/manipulator.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);


}
Manipulator2D::~Manipulator2D()
{
}

void Manipulator2D::updateLeftHandInput(Pose &pose)
{
	update(pose);
}


void Manipulator2D::updateRightHandInput(Pose &pose)
{
	update(pose);
}	

void Manipulator2D::updateBothHandInput(Pose &pose)
{
	math::Vec2f right_hand_coords = RightHandPointer->getWindowCoordinates();
	math::Vec2f left_hand_coords = LeftHandPointer->getWindowCoordinates();

	// incremental calculations

	// ROTATION		
	Rotation = getCurrentHandAngle(left_hand_coords, right_hand_coords) - LastRotation;
	
	// SCALING
	Scale = (right_hand_coords - left_hand_coords).getLength() / LastScale; 
		
		
	//POSITIONING
	RotationCenter = 0.5 * (right_hand_coords + left_hand_coords);
	Translation = RotationCenter - LastTranslation;

	setTransformationMatrix();
}
	

void Manipulator2D::update(Pose &pose)
{
	math::Vec2f right_hand_coords = RightHandPointer->getWindowCoordinates();
	math::Vec2f left_hand_coords = LeftHandPointer->getWindowCoordinates();

	// always gather the system parameters on passive update
	LastRotation = getCurrentHandAngle(left_hand_coords, right_hand_coords);
	LastScale = (right_hand_coords - left_hand_coords).getLength();
	LastRotationCenter = 0.5 * (right_hand_coords + left_hand_coords);
	LastTranslation = LastRotationCenter;
	LastTransformationMatrix = TransformationMatrix;
}	

void Manipulator2D::render()
{
}

void Manipulator2D::setTransformationMatrix()
{
	TransformationMatrix = math::Matrix44f::Identity();
	TransformationMatrix.translate(RotationCenter.x, RotationCenter.y, 0.0f);
	TransformationMatrix.rotateZ(Rotation);
	TransformationMatrix.scale(Scale);
	TransformationMatrix.translate(-RotationCenter.x, -RotationCenter.y, 0.0f);
	TransformationMatrix.translate(Translation.x, Translation.y, 0.0f);

	TransformationMatrix = LastTransformationMatrix * TransformationMatrix;
}

math::Matrix44f& Manipulator2D::getTransformationMatrix()
{
	return TransformationMatrix;
}

float Manipulator2D::getCurrentHandAngle(math::Vec2f& left_hand_coords, math::Vec2f& right_hand_coords)
{
	float angle = atan2f( abs(right_hand_coords.y - left_hand_coords.y),  abs(right_hand_coords.x - left_hand_coords.x));
				
	math::Vec2f temp = right_hand_coords - left_hand_coords;

	// in which quadrant temp is lying
	if ( temp.x >= 0 ) // I or IV
	{
		return (temp.y > 0) ? (angle) : (-angle);
	}
	else // II or III
	{
		return (temp.y > 0) ? (3.14159f - angle) : (3.14159f + angle);
	}
}




math::Vec2f Manipulator2D::WorldToLocalCoords(math::Vec2f & coords)
{
	math::Matrix44f temp = getTransformationMatrix();
	temp.invert();
	return math::Vec2f(temp.m[0][0] * coords.x + temp.m[1][0] * coords.y + temp.m[3][0], temp.m[0][1] * coords.x + temp.m[1][1] * coords.y + temp.m[3][1]);	
}