/*
This tool provides scale, rotation and translation of canvas.

*/
#pragma once
#include "Tool.h"
#include "Camera.h"
#include "math/Math.h"
#include "math\Vec2f.h"
#include "KinectPointer.h"

class Manipulator2D : public Tool
{
public:

	Manipulator2D(KinectPointer* left_hand, KinectPointer* right_hand);
	~Manipulator2D();

	void updateLeftHandInput(Pose &pose); 
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose); 

	void update(Pose &pose);
	void render();
	void undo(){}
	void redo(){}

	int getTextureID(){return TextureID;};

	// returns transformation matrix (used as opengl modelview matrix)
	math::Matrix44f& getTransformationMatrix();
	// calculates local coordinates from given world coordinates
	math::Vec2f WorldToLocalCoords(math::Vec2f & coords);

private:
	// calculates angle between hands in x-y-plane
	float getCurrentHandAngle(math::Vec2f& left_hand_coords, math::Vec2f& right_hand_coords);
	void setTransformationMatrix();

	// manipulation parameters
	math::Vec2f RotationCenter;
	math::Vec2f Translation;
	float Rotation;
	float Scale;
	// manipulation parameters of last step
	math::Vec2f LastRotationCenter;
	math::Vec2f LastTranslation;
	float LastRotation;
	float LastScale;

	// pointers to hand pointers
	KinectPointer* LeftHandPointer;
	KinectPointer* RightHandPointer;

	// last and current transformation matrices
	math::Matrix44f LastTransformationMatrix;
	math::Matrix44f TransformationMatrix;
	
	// icon texture
	int TextureID;
};