/*
This tool provides camera manipulation in orbit style, i.e. we set azimuth, elevation and distance of the camera to the center (look at point) of an imaginary sphere.
*/
#pragma once
#include "Tool.h"
#include "math\Math.h"
#include "Camera.h"


class OrbitManipulator : public Tool
{
public:

	OrbitManipulator();
	OrbitManipulator(Camera *camera);
	~OrbitManipulator();

	void init(math::Vec3f look_at, float azimuth, float elevation, float distance, float scaling, Camera *camera);

	void updateLeftHandInput(Pose &pose); 
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose); 

	void update(Pose &pose);
	void render();
	void undo(){}
	void redo(){}

	// return texture id for editor icon
	int getTextureID(){return TextureID;}
	
	// calculates a new camera transformation matrix
	void updateCamera();
	// assigns camera object
	void setCamera(Camera *camera);
	
	// different system transformations
	void pan(float delta_x, float delta_y);
	void rotate(float delta_azimuth, float delta_elevation);
	void zoom(float delta_distance);
	void setLookAt(float x, float y, float z);

	// the transformation matrix
	math::Matrix44f TransformationMatrix;
	
	// system parameters
	math::Vec3f LookAt;
	float Azimuth, Elevation;
	float  Distance;
	float Scaling;

	// pointer to assigned camera
	Camera *pCamera;
	// maintain pose from last iteration step
	Pose LastPose;

	// tool icon texture
	int TextureID;
};

