/*
This tool provides local smoothing of a grid mesh. A point on the mesh is smoothed out by the affine combination 
of its height and the height centroid of the adjacent grid points.
The closed right hand invokes the tool and sets its position. The left hand adjusts size and strength of the tool.

*/
#pragma once

#include "Tool.h"
#include "math\Vec3f.h"
#include "KinectPointer.h"
#include "KinectTerrain\GridMesh.h"


class SmoothBrush: public Tool
{
public:
	SmoothBrush(KinectPointer* right_hand, GridMesh* mesh);
	virtual ~SmoothBrush(void);

	void updateLeftHandInput(Pose &pose);
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose);

	void update(Pose &pose);
	void render();
	void undo(){}
	void redo(){}

	// return texture id for editor icon
	int getTextureID(){return TextureID;}



private:
	// sets the tool parameters
	void setRadiusAndStrength(Pose& pose);
	// actual tool operation
	void processMesh();

	// pointer to hand pointer
	KinectPointer* RightHandPointer;
	// pointer to mesh
	GridMesh* Mesh;
	// maintain pose from last iteration step
	Pose LastPose;

	// brush parameters
	float BrushRadius;
	float BrushStrength;
	// show brush if inside grid
	bool showBrush;
	// local coords of brush tip
	math::Vec3f BrushCoordinates;
	
	// tool icon texture
	int TextureID;
};

