#include "StrokeEraser2D.h"
#include "SOIL.h"


StrokeEraser2D::StrokeEraser2D(std::vector<Stroke>* strokes, int* selected_stroke_id)
{
	Strokes = strokes;
	SelectedStrokeID = selected_stroke_id;
	TextureID = SOIL_load_OGL_texture("../images/trashtool.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
}
StrokeEraser2D::~StrokeEraser2D()
{
}

void StrokeEraser2D::updateLeftHandInput(Pose &pose)
{
}


void StrokeEraser2D::updateRightHandInput(Pose &pose)
{
	if(*SelectedStrokeID != -1)
	{
		// if we have a valid stroke id we try to get the corresponding stroke in the list
		for(unsigned int i = 0; i < Strokes->size(); i++)
		{
			if( (*Strokes)[i].ID == (*SelectedStrokeID) )
			{
				// save an undo information -> copy the stroke
				UndoList.push_back((*Strokes)[i]);
				// update the undo/redo manager
				UndoRedoManager::checkIn(this);
				// clear the redo list
				RedoList.clear();

				// erase the stroke
				Strokes->erase(Strokes->begin() + i);
				break;
			}
		}
	}
}	

void StrokeEraser2D::updateBothHandInput(Pose &pose)
{
	updateRightHandInput(pose);
}
	

void StrokeEraser2D::update(Pose &pose)
{
}	

void StrokeEraser2D::render()
{
}



void StrokeEraser2D::undo()
{
	if(UndoList.size() > 0)
	{
		//get the last element and push it back on the stroke stack
		Strokes->push_back(UndoList.back());
		// also push it on the redo stack
		RedoList.push_back(UndoList.back());
		//pop it from the undo stack
		UndoList.pop_back();
	}
}

void StrokeEraser2D::redo()
{
	if(RedoList.size() > 0)
	{
		//clear stroke with same id from the stroke stack
		for (unsigned int i = 0; i < Strokes->size(); i++)
		{
			if((*Strokes)[i].ID == RedoList.back().ID)
			{
				Strokes->erase(Strokes->begin() + i);
				break;
			}
		}

		// save the info in the undo list
		UndoList.push_back(RedoList.back());

		//pop the element from the redo stack
		RedoList.pop_back();
	}
}