/*
With this tool we can "draw" hills and dips into virtual landscapes based on grid meshes.
The closed right hand invokes the tool and sets its position. The closed left hand adjusts size and shape of the resulting hills/dips.
*/
#pragma once

#include "Tool.h"
#include "math\Vec3f.h"
#include "KinectPointer.h"
#include "KinectTerrain\GridMesh.h"


class TerraBrush : public Tool
{
public:
	TerraBrush(KinectPointer* right_hand, GridMesh* mesh);
	virtual ~TerraBrush(void);

	void updateLeftHandInput(Pose &pose);
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose);

	void update(Pose &pose);
	void render();
	void undo(){}
	void redo(){}

	// return texture id for editor icon
	int getTextureID(){return TextureID;}



private:
	// sets the tool parameters
	void setRadiusAndCurvature(Pose& pose);
	// actual tool operation
	void processMesh();
	
	// pointer to hand pointer
	KinectPointer* RightHandPointer;
	// pointer to mesh
	GridMesh* Mesh;
	// maintain pose from last iteration step
	Pose LastPose;

	
	// brush parameters
	float BrushRadius;
	float BrushCurvature;
	// show brush if inside grid
	bool showBrush;
	// local coords of brush tip
	math::Vec3f BrushCoordinates;
	
	// tool icon texture
	int TextureID;
};

