#include "TerraColorizer.h"
#include "Tools\Colorpicker.h"
#include "SOIL.h"
#include <iostream>


TerraColorizer::TerraColorizer(KinectPointer* right_hand, GridMesh* mesh, WindowProperties* window_props)
{
	RightHandPointer = right_hand;
	BrushRadius = 4.0f;
	BrushAlpha = 1.0f;
	Mesh = mesh;
	showBrush = false;
	IconTextureID = SOIL_load_OGL_texture("../images/brushtool.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	TextureID = SOIL_load_OGL_texture("../images/colorbrush_half_sharp.png", SOIL_LOAD_RGBA, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS);
	WindowProps = window_props;
}



TerraColorizer::~TerraColorizer(void)
{
}


void TerraColorizer::updateLeftHandInput(Pose &pose)
{
	setRadiusAndAlpha(pose);
	LastPose = Pose(pose);
}

void TerraColorizer::updateRightHandInput(Pose &pose)
{
	processMesh();
	LastPose = Pose(pose);
}

void TerraColorizer::updateBothHandInput(Pose &pose)
{
	setRadiusAndAlpha(pose);
	processMesh();
	LastPose = Pose(pose);
}

void TerraColorizer::update(Pose &pose)
{
	showBrush = (RightHandPointer->isActive())?(Mesh->ScreenToLocalCoords(RightHandPointer->getWindowCoordinates(), BrushCoordinates, false)):(false);
	LastPose = Pose(pose);
}

void TerraColorizer::render()
{
	// render brush
	if (showBrush)
	{
		GLUquadric* quadric = gluNewQuadric();
		glPushMatrix();
		glTranslatef(BrushCoordinates.x, BrushCoordinates.y + .1f, BrushCoordinates.z);

	
		glColor3f(1.f, 1.f - BrushAlpha, 1.f - BrushAlpha);
		
		
		gluSphere(quadric,0.2,10,10);
		glRotatef(-90, 1.0f, 0.0f, 0.0f);
		
		gluDisk(quadric, BrushRadius - 0.3f, BrushRadius + 0.3f, 20, 1);


		gluDeleteQuadric(quadric);
		glPopMatrix();
	}
}

void TerraColorizer::setRadiusAndAlpha(Pose& pose)
{
	float diff_x = pose.LeftHandPosition.x - LastPose.LeftHandPosition.x;
	float diff_y = pose.LeftHandPosition.y - LastPose.LeftHandPosition.y;

	if ( abs(diff_x) > abs(diff_y))
	{
		BrushRadius += 50 * diff_x;
		BrushRadius = max(0.0f, BrushRadius);
	}
	else 
	{
		BrushAlpha += diff_y;
		BrushAlpha = max(0.1f, BrushAlpha);
		BrushAlpha = min(1.0f, BrushAlpha);

	}
}

void TerraColorizer::processMesh()
{
	showBrush = Mesh->ScreenToLocalCoords(RightHandPointer->getWindowCoordinates(), BrushCoordinates, false);


	if(showBrush)
	{
		glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glDisable(GL_LIGHTING);
		glDisable(GL_DEPTH_TEST);
		glEnable(GL_TEXTURE_2D);
		glMatrixMode(GL_PROJECTION);
		glPushMatrix(); //save projection transformations
		glLoadIdentity();
		float window_width = (float) WindowProps->WindowWidth;
		float window_height = (float) WindowProps->WindowHeight;
		glOrtho(0.0, window_width, 0.0, window_height,-1.0,1.0);

		glMatrixMode(GL_MODELVIEW);
		glPushMatrix(); //save modelview transformations
		glLoadIdentity();

		// render mesh texture in screen space
		Mesh->drawColorMap();
	
	
		glPushMatrix();
		// draw brush texture
		float width = (float) Mesh->getWidth();
		float height = (float) Mesh->getLength();
		glTranslatef(BrushCoordinates.x/width * 512, BrushCoordinates.z/height * 512, 0.0f); // <- 512x512 is the size of the mesh texture
		glScalef(BrushRadius/width * 512, BrushRadius/height * 512, 1.0f); 

		glColor4f(Colorpicker::CurrentColor.x, Colorpicker::CurrentColor.y, Colorpicker::CurrentColor.z, BrushAlpha);
		glBindTexture(GL_TEXTURE_2D, TextureID);
		glBegin(GL_QUADS);
			glTexCoord2f(0.0, 0.0);		glVertex3f(-2.0, -2.0, 0.0);
			glTexCoord2f(0.0, 1.0);		glVertex3f(-2.0, 2.0, 0.0);
			glTexCoord2f(1.0, 1.0);		glVertex3f(2.0, 2.0, 0.0);
			glTexCoord2f(1.0, 0.0);		glVertex3f(2.0, -2.0, 0.0);
		glEnd();
		glPopMatrix();

		// render result into mesh texture
		glBindTexture(GL_TEXTURE_2D, Mesh->getTextureID());
		glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 0, 0, 512, 512, 0);

		glEnable(GL_DEPTH_TEST);
		glPopMatrix(); //restore modelview transformations
		glMatrixMode(GL_PROJECTION);
		glPopMatrix(); //restore projection transformations
		glMatrixMode(GL_MODELVIEW);
	}

}