/*
This class provides the coloration of a grid mesh over a texture copy operation.  
Invoke the tool and set its position with the closed right hand. Adjust size and transparency with the closed left hand.
*/
#pragma once

#include "Tool.h"
#include "math\Vec3f.h"
#include "KinectPointer.h"
#include "KinectTerrain\GridMesh.h"
#include "WindowProperties.h"


class TerraColorizer : public Tool
{
public:
	TerraColorizer(KinectPointer* right_hand, GridMesh* mesh, WindowProperties* window_props);
	virtual ~TerraColorizer(void);

	void updateLeftHandInput(Pose &pose);
	void updateRightHandInput(Pose &pose);
	void updateBothHandInput(Pose &pose);

	void update(Pose &pose);
	void render();
	void undo(){}
	void redo(){}

	// return texture id for editor icon
	int getTextureID(){return IconTextureID;}

	

private:
	// sets tool parameters
	void setRadiusAndAlpha(Pose& pose);
	// actual tool operation
	void processMesh();

	// pointer to hand pointer
	KinectPointer* RightHandPointer;
	// pointer to mesh
	GridMesh* Mesh;
	// pointer to window properties (used in render method)
	WindowProperties* WindowProps;
	// maintain pose from last iteration step
	Pose LastPose;
	
	// brush parameters
	float BrushRadius;
	float BrushAlpha;
	// show brush if inside grid
	bool showBrush;
	// local coords of brush tip
	math::Vec3f BrushCoordinates;

	// tool icon texture
	int IconTextureID;
	// brush texture
	int TextureID;

};

